# file: bindmat.awk
# replacement for compute-binding-matrix
# written by: John M. Gravley
#
# This file expects the XXX.sorted-useinfo file as input.
# It outputs a three tuple of the form 
#	`(func-name func-name count-of-references)
# It takes into account either func-i func-j var or
#   func-j func-i var in the input stream.
# It ignores var in either case.
# The output still needs to be sorted, so a call would look like:
#	nawk -f bindmat.awk XXX.sorted-useinfo | sort > XXX.bind

# This program takes advantage of awk's associative array indexing.
# *** THIS PROGRAM REQUIRES NAWK ***

{
	if (($1,$2) in count)		# if already have it as i,j
		count[$1,$2]++
	else if (($2,$1) in count)	# if already have it as j,i
		count[$2,$1]++
	else				# first time
		count[$1,$2] = 1
}

END {
	for (k in count) {
		split(k, x, SUBSEP)
		print x[1], x[2], count[k]
	}
}
