# file: makemat.awk
# replacement for make-binding-matrix.awk
# written by: John M. Gravley
#
# This file expects the XXX.label and XXX.bind files as input.
# It outputs a binding matrix of the form b(i,j) = count.
# A call would look like:
#	nawk -f makemat.awk XXX.labels XXX.bind > XXX.matrix

# This program takes advantage of awk's associative array indexing.
# *** THIS PROGRAM REQUIRES NAWK ***

BEGIN {		# read in all of the labels
	while (getline < ARGV[1] > 0) {
		name[$1] = 1
	}

	# reverse the order of output -- by default is backwards (i.e., sorted descending)
      	for (i in name) {
		name2[i] = name[i]
	}

}

{		# now get the counts
	count[$1,$2] = $3
}

END {		
	# debug = 1	# uncomment for debug output

	if (debug == 1) {
		for (i in name2) {
			printf "%s\n", i
		}
	}

		# and finally output the matrix
	for (i in name2) {
		for (j in name2) {
			if ((i,j) in count)
				printf "%d ", count[i,j]
			# if only want bottom triangle of symmetric
			#    matrix then
			#	comment out next two program lines
			#	and then filter out first lines
			#	that contain all zeroes
			else if ((j,i) in count)
				printf "%d ", count[j,i]
			else
				printf "0 "
		}
		printf "\n"
	}
}
