#include <stdio.h>

#include "bclust.h"
#include "tree.h"
#include "binding.h"
#include "distance.h"
#include "suppvect.h"
#include "clustvect.h"

char *suppress_list = NULL;
int dump=-1;


void S_bclust(int *dimp,
              int *bind,
              int *merge,
              float *height,
              int *order,
              int *debugp,
              int *modep,
              int *tracep,
	      int *clusterp,
              char **reorderp
             )
{
  int dim;
  int debug;
  int mode;
  int trace;
  int cluster;
  int reorder;

  Options state;

  Tree_ptr tree;
  Bind_mat bind_mat;

  dim= *dimp ;
  debug= *debugp;
  mode= *modep;
  trace= *tracep;
  cluster= *clusterp;
  reorder= (int) **reorderp;

  /* transfer option information to state structure */
  setDebugOption(state, debug);
  setTraceOption(state, trace);
  setModeOption(state, mode);
  setClusterOption(state, cluster);
  setReorderOption(state, reorder);


  if (interfaceTest(state)) fprintf(stderr, "Before initialize tree\n");

  /* create a tree structure .. by transfering the addresses of 
     various input vectors to the structure */

  tree = sCreateTreeStr(merge, height, order, dim);
  bind_mat = sCreateBindingMatrix(bind, dim);

  if (interfaceTest(state)) fprintf(stderr, "After initialize tree\n");

  /* call bclust  */
   bclust(dim, bind_mat,  tree,  state);

  
}




void bclust(int dim,
		Bind_mat bind, 
		Tree_ptr tree,
		Options state)

{
	/* matrices */
	Dist_mat dist;

	/* for various computations */
	I_vect suppress_vector;
	Cl_vect merge_pos, clust_vector;

	/* variables */
	int no_of_clusts;
	float min_dist;

	/********************INITIALIZE*************************/

	no_of_clusts = 0;

	dist = createDistanceMatrix(dim);
	suppress_vector = createSuppressVector(dim);
	clust_vector = createClustVector(dim);	

	resetClustVector(clust_vector, ABSURD);
	resetSuppressVector(suppress_vector);

        if (suppress_list !=NULL) 
	{
	    stringSuppressVector(suppress_vector, suppress_list);
	    no_of_clusts = countSuppressed(suppress_vector);
        }

	/*** ASSERTION ******/
       if (no_of_clusts != countSuppressed(suppress_vector)) 
       {
          myError(" ****** Assertion violated before entering bclust() **** \n");
       }


	if (debugOption(state))
		printf("Debug : %d Mode : %d Trace : %d :Cluster : %d\n", 
			debugOption(state),modeOption(state), 
			traceOption(state), clusterOption(state));

	while (no_of_clusts < dim-1)
	{  
             if (traceOption(state)) 
	     {
                 printf("Trace: %d (Dim: %d)\n", no_of_clusts, dim);
                 fflush(stdout);
             }
             if ((dump == 0) ||   (dump == no_of_clusts)) 
	     {
		 printBindMatrix(bind, suppress_vector);
		 myError("Exiting after dumping matrix\n");
             }

	      min_dist = computeDistance(state, 
					no_of_clusts,
					clust_vector,
				 	suppress_vector, 
					bind, 
					dist);

	      no_of_clusts = traverseAndUpdate(dist, 
						min_dist,
						suppress_vector,
						clust_vector,
						tree,
						state);



	      /* now for STATUS report */
	      if (debugOption(state))
	      {
		 printf("\n\n\n");
		 printBindMatrix(bind, suppress_vector);
		 printDistMatrix(dist, suppress_vector);
		 printTreeStr(tree);
		 printIntVector(tree->merge_pos, " Merge_pos ");
	         printSuppressVector(suppress_vector); 
	      }

	      if (traceOption(state))
	      {
		 printf("\n");
		 printf("CLUSTERS FORMED = %d .\n", no_of_clusts);
	      }


	      /* CALCULATE NEW BINDINGS AND then UPDATE SUPPRESS VECT */

	      computeBinding(suppress_vector, clust_vector, bind);

	      updateSuppressVector(suppress_vector, clust_vector);

             if (no_of_clusts != countSuppressed(suppress_vector))
	     {
	        myError(" ****** Assertion violated in bclust() **** \n");
	     }

	}	/* end of main while loop */

        switch (reorderOption(state)) 
	{
	        case 'l':
		case 0:	    makeOrderFromMerge(tree);
	  	     break;
		case 't': makeOrderVector(tree);      /* 't' = 116 */
	  	     break;
		default:
	   	        fprintf(stderr, "Unrecoginized option (%c) for reorder... 
			ignored \n", reorderOption(state));
	  	        makeOrderFromMerge(tree);

        }

}



