#include <stdio.h>
#include "clustvect.h"
#include "errors.h"
#include "globals.h"

/* prototypes */
void setClustVector(Cl_vect clust, int index, int val);

char *calloc();

Cl_vect createClustVector(int dim)
{
	Cl_vect clust;

	clust = emalloc(sizeof(Clust_str), "Clust Structure");
	clust->c_vect = (int *) ecalloc(dim, sizeof(int), "Clust vect");
	clust->dim_c = dim;
	resetClustVector(clust, ABSURD);
	return clust;
}


void destroyClustVector(Cl_vect clust_vect)
{
	free(clust_vect->c_vect);
	free(clust_vect);
}

/* updateClustVector returns cluster */

int updateClustVector(Cl_vect clust_vect, 
			int row, 
			int col, 
			int cluster)
{
	/* update clust_vect now, first row then col */
	/* remember, row is always less than col */
	/* check the case for creating a new cluster first */

 	int tempr, tempc, min;
			
	tempr = tempc = 0;

	if (!isClustered(clust_vect, row) &&
	    !isClustered(clust_vect, col))
	{
	   cluster++;
	   setClustVector(clust_vect, row, cluster);
	   setClustVector(clust_vect, col, cluster);
	}
	else if (isClustered(clust_vect, row) &&
		isClustered(clust_vect, col))
	{
	    /* set the lower cluster in "upper" location */

	    tempr = getCluster(clust_vect, row);
	    tempc = getCluster(clust_vect, col);
	    if (tempr < tempc)
	    {
		min = tempr;
		setClustVector(clust_vect, col, min);
		/* set takes care of resetting all old values */
	    }
	    else
	    {
		min = tempc;
		setClustVector(clust_vect, row, min);
	    }

	  } 		/* end of elseif */
	else		/* one of them is clustered */
	{
	   if (isClustered(clust_vect, row))
	   {
	      tempr = getCluster(clust_vect, row);
	      setClustVector(clust_vect, col, tempr);
	   }
	   else
	   {
	      tempc = getCluster(clust_vect, col);
	      setClustVector(clust_vect, row, tempc);
	   }	
	}
	
	return cluster;
}



int getClustHead(Cl_vect clust_vect, int member)
{
	int i;
	int to_look_for;

	/* 
	traverse the clust_vect and find the header for
 	that element's cluster 
	*/

	to_look_for = getCluster(clust_vect, member);

	if (to_look_for == ABSURD)		/* no cluster */
	   return ABSURD;

	/*
	the first entry in clust_vect with value equal to
	to_look_for has to be the cluster header
	stop at the first value, return position
	*/

	for (i =1; (i < member) && 
		   (getCluster(clust_vect, i) != to_look_for) ; i++)
		;

	return i;
}



int isClustHead(Cl_vect clust, int member)
{
	/* from clust check if the member is the header */

	int i;
	int to_look_for;
	int found_first;

	found_first = TRUE;
	to_look_for = getCluster(clust, member);

	for (i = 1; (i < member) && (found_first); i++)
	{
	    if (getCluster(clust, i) == to_look_for)
		found_first = FALSE; 
	}
	return found_first;	    
}



int isClustered(Cl_vect clust_vect, int member)
{
	if (getCluster(clust_vect, member) != ABSURD)
	   return TRUE;
	else
	   return FALSE;
}



int inSameCluster(Cl_vect clust_vect, int member1, int member2)
{
	if (getCluster(clust_vect, member1) == 
	               getCluster(clust_vect, member2))
	   return TRUE;
	else
	   return FALSE;
}




void setClustVector(Cl_vect clust, int index, int val)
{
	int i;
	int old_clust;
	int dim = clust->dim_c;

	old_clust = getCluster(clust,index);

	/* if not previously clustered, put the new value there */
	/* otherwise, replace all old values by the new value */

	if (old_clust == ABSURD)
	{
	   clust->c_vect[index-1] = val;
	}
	else
	{
	   clust->c_vect[index-1] = val;
	   for (i = 1; i <= dim; i++)
	   {
		if (getCluster(clust, i) == old_clust)
		   clust->c_vect[i-1] = val;
	   }
	}

	
}


void resetClustVector(Cl_vect clust, int val)
{
	int i;
	int dim = clust->dim_c;

	for (i = 0; i < dim; i++)
	    clust->c_vect[i] = val;
}


/* just returns the contents */
int getCluster(Cl_vect clust_vect, int member)
{
	return clust_vect->c_vect[member-1];
}


void printClustVector(Cl_vect clust, char * vect_name)
{
	int i;
	int val;
	int dim = clust->dim_c;

	printf("Printing the vector ");
	printf("%s \n", vect_name);
	for (i = 1; i <= dim; i++)
	{
	    val = getCluster(clust, i);
	    printf("%d ", val);
	}
	printf("\n");
}



		    

	

