#include <stdio.h>
#include "distance.h"
#include "globals.h"

/* prototypes */
void setDistValue(Dist_mat dist, int row, int col, float value);
float getDistValue(Dist_mat dist, int row, int col);

Dist_mat createDistanceMatrix(int dim)
{
	int i;
	Dist_mat dist;

	dist = (Dist_mat) emalloc(sizeof(Dist_str), "Distance structure");

	dist->d_mat = (Two_dim) ecalloc(dim, sizeof(float *), "Dist Mat");

	for (i = 1; i <= dim; i++)
	{
	    dist->d_mat[i-1] = (float *) ecalloc(dim, sizeof(float), "dist mat");
	}
	dist->dim_d = dim;
	return dist;
}


Dist_mat sCreateDistanceMatrix(float * dist_space, int dim)
{
	int i;
	Dist_mat dist;

	dist = (Dist_mat) emalloc(sizeof(Dist_str), "Distance structure");
	
	dist->d_mat = (Two_dim) ecalloc(dim, sizeof(float *), "Dist Mat");
	for (i =1; i <= dim; i++)
	{
	    dist->d_mat[i-1] = (float *) (dist_space + (i-1)*(dim));
	}
	dist->dim_d = dim;
	return dist;
}

void sFreeDistanceMatrix(Dist_mat dist)
{
	free(dist->d_mat);
	free(dist);
}


float getDistValue(Dist_mat dist_mat, int row, int col)
{
	if (row == col)
	   return ABSURD;
	else
	   return dist_mat->d_mat[row-1][col-1];
}


void setDistValue(Dist_mat dist_mat, int row, int col, float value)
{
	dist_mat->d_mat[row-1][col-1] = value;
}

void resetDistanceMatrix(Dist_mat dist, float value)
{
	int row, col;

	for (row = 1; row <= dist->dim_d; row++)
	{
	    for (col = 1; col <= dist->dim_d; col++)
	    {
		setDistValue(dist, row, col, value);
	    }
	}
}


float computeDistance(Options state,  
			int no_of_clusts, 
			Cl_vect clust_vect, 
		     	I_vect suppress,
			Bind_mat bind, 
			Dist_mat dist_mat)

{
	int row,col;
	float distance;		/* to be calculated */
	float min_dist;
	float numerator, denominator;
	I_vect bind_sum;	/* to find the total bindings */
	int nodes_present;	/* how many elements left */
	int nodes_minus_1;	/* used for optimization */
	int common_bind;	/* bindings betn two given nodes */
	int row_sum, col_sum;	/* for binding sums */

	float ** dist = dist_mat->d_mat;
	int dim = dist_mat->dim_d;

	min_dist = INFINITY;
	nodes_present = dim - no_of_clusts;	/* check that */
	nodes_minus_1 = nodes_present - 1;

	bind_sum = createIntVector(dim);

	resetClustVector(clust_vect, ABSURD);
	resetIntVector(bind_sum, ABSURD);
	
	computeBindingSum(bind, suppress, bind_sum);

	for (row = 1; row < dim; row++)
	{
	    if (isSuppressed(suppress, row))
		continue;
	    for (col = row+1; col <= dim; col++)
	    {
		if (isSuppressed(suppress, col))
		   continue;
		
		common_bind = getBindValue(bind, row, col);
		row_sum = getBindingSum(bind_sum, row);
		col_sum = getBindingSum(bind_sum, col);

		/* MODE WILL  DETERMINE THE ALGORITHM */
		switch (modeOption(state))
		{
		     case 1:	/* Expected binding */
		     {
			numerator = (float)(row_sum + col_sum - common_bind) /
						nodes_minus_1;
			denominator = (float) (common_bind);

			/* check the 0/0 case first , then do the needful */
			if ((numerator == 0.0) && (denominator == 0.0))
		   	   fprintf(stderr, "CAUTION: Distance in 0/0 form\n");

			if (denominator != 0.0)
			distance = numerator / denominator;
			else 
		   	   distance = INFINITY;

			break;
		     }
		     case 2:		/* recomputed binding */
		     {
			numerator = (float)(row_sum + col_sum - 2*common_bind);
			denominator = (float)(row_sum + col_sum - common_bind);
			
			/* check the 0/0 case first , then do the needful */
			if ((numerator == 0.0) && (denominator == 0.0))
		   	   fprintf(stderr, "CAUTION: Distance in 0/0 form\n");

			if (denominator != 0.0)
			distance = numerator / denominator;
			else 
		   	   distance = INFINITY;

			break;
		
		     }		    

		}
		setDistValue(dist_mat, row, col, distance);
		/* make it symmetric */
		setDistValue(dist_mat, col, row, distance);

		if (distance <= min_dist) 
		   min_dist = distance;

	    }	/* col for loop */
	}	/* row for loop */
	
	destroyIntVector(bind_sum);	/* free memory */
	return min_dist;
}
		

/* 
traverseAndUpdate will traverse the distance matrix and update clust_vect,
merge_pos, and merge_vect. It returns the no. of new+old clusters indicated
by the dist matrix . All these might, at a later stage, merge into 
a single cluster. Basically, this value is the same as the total number
of occurances of min_dist in the dist matrix, plus the previous value.
Also updates order_vector
Affirm: returned value is the same as number of new entries in merge_vect. 
*/

int traverseAndUpdate(Dist_mat dist_mat,
		      float min_dist, 
		      I_vect suppress,
		      Cl_vect clust_vect, 
		      Tree_ptr tree, 
		      Options state)
{
	int row,col;
	int cluster;		/* the real (big) cluster */
	int no_of_clusts;	/* return value */
	int new_cluster;	/* boolean, whether new cluster formed or not */
	int dim = dist_mat->dim_d;
	
	cluster = 0;
	resetClustVector(clust_vect, ABSURD);
	

	for (row = 1; row < dim; row++)
	{
	    if (isSuppressed(suppress, row))
		continue;
	    
	    for (col = row+1; col <= dim; col++)
	    {
		new_cluster = FALSE;	/* INITIALIZE */
		if (isSuppressed(suppress,col))
		   continue;

		if (getDistValue(dist_mat,row,col) == min_dist)
		{
		   /* THE GAME BEGINS */
		   /* the cluster switch will tell which definition of
		   cluster to choose. cluster = 1, means that a cluster is (ONE)
		   pair (i.e. ONLY two elements). cluster = 2 implies that
		   a cluster may have any number of elements.
		   */
		   switch (clusterOption(state))
		   {
			case 1:
				/*
				New clusters can be formed if
				1. i->o, j->0, ie, both are independent units 
				*/
				if ( !isClustered(clust_vect, row) &&
				     !isClustered(clust_vect, col))
				   new_cluster = TRUE;
				break;
				
		   	case 2:
		   		/*
		   		First see if new clusters are REALLY formed. 
				New means
		   		(1) both are independent elements, or
		   		(2) Either one is independent (at least one)
		   		(3) Both are in different clusters
		   		*/
		   		if ( isClustered(clust_vect, row) &&
			     	     isClustered(clust_vect, col) &&
			     	     inSameCluster(clust_vect, row, col))
			   	   new_cluster = FALSE;
				else
				   new_cluster = TRUE;
				break;
			default:
				break;
		   }			

		   if (new_cluster)
		   {
		      cluster = updateClustVector(clust_vect,
						  row, col, 
						  cluster);
		      updateTreeStr(tree,
				    row, col,
				    min_dist);
		   }

		}
	    }
	}
	no_of_clusts = getClustFromTree(tree);
	return no_of_clusts;

}


void printDistMatrix(Dist_mat dist, I_vect suppress)
{
	int row, col;
	float val;
	int dim = dist->dim_d;

	printf("Printing Matrix ....\n");

	for (row = 1; row <= dim; row ++)
	{
	    if (isSuppressed(suppress, row))
		continue;
	    for (col = 1; col <=dim; col++)
	    {
		if (isSuppressed(suppress, col))
		   continue;
		val = getDistValue(dist, row, col);
		printf("%4.1f ", val);
	    }
	    printf("\n");
	}
}		      


void S_computeDistance(int *dimp,
			int *bind_mat,
			int *modep,
			float *dist_mat,
			int *debugp)
{
	Options state;
	int dum_no_of_cls;
	Cl_vect dum_cl_vect;
	I_vect dum_suppress;
	Bind_mat bind;
	Dist_mat dist;
	int debug = *debugp;
	int mode = *modep;
	int dim = *dimp;
	int trace, cluster, reorder;
	int min_dist;
	trace = 0;
	cluster = 1;
	reorder = 0;

	state = setOptions(debug, trace, mode, cluster, reorder);
	dum_no_of_cls = 0;
	dum_cl_vect = createClustVector(dim);
	dum_suppress = createIntVector(dim);
	dist = sCreateDistanceMatrix(dist_mat, dim);
	bind = sCreateBindingMatrix(bind_mat, dim);
	
	min_dist = computeDistance(state,
				   dum_no_of_cls,
				   dum_cl_vect,
				   dum_suppress,
				   bind,
				   dist);
	
	if (debugOption(state))
	   printDistMatrix(dist, dum_suppress);
		
}		   

	

	
		

	



	

	
