#include <stdio.h>
#include "goodness.h"
#include "globals.h"

Good_ptr createGoodnessStr(int dim)
{
	Good_ptr good;

	good = (Good_ptr) emalloc(sizeof(Good_str), "Goodness structure");
	good->ht_arr = createRealVector(dim);
	good->sys_dist = createDistanceMatrix(dim);
	resetDistanceMatrix(good->sys_dist, R_ABSURD);
	good->dim_g = dim;
	return good;
}

Good_ptr sCreateGoodnessStr(float * pair_dist, int dim)
{
	Good_ptr good;

	good = (Good_ptr) emalloc(sizeof(Good_str), "Goodness structure");
	good->ht_arr = createRealVector(dim);
	good->sys_dist = sCreateDistanceMatrix(pair_dist, dim);
	resetDistanceMatrix(good->sys_dist, R_ABSURD);
	good->dim_g = dim;
	return good;
}

int traverseForAncestor(Tree_ptr tree,
				Good_ptr good,
                		int root, 
                		int left_num)
{
	int left_son_span;
	int right_son_span;

   	if (isleaf(tree, root)) 
	{
           return left_num+1;
   	} 
	else 
	{
       	    left_son_span = traverseForAncestor(tree, good, leftnode(tree, root),
			   		 	left_num);
       	    right_son_span = traverseForAncestor(tree, good, rightnode(tree, root),
						 left_son_span);

            computePairwiseDistance(good, tree, root, left_num, 
				    left_son_span, right_son_span);
            return right_son_span;
   	}
}

void S_pairwiseDistance( int *dimp, 
			 int *merge,
			 float *height,
			 int *order,
			 float *pair_dist,
			 int *debugp
		       )
{
	int dim;
	int debug;
	Options state;
	Tree_ptr tree;
	Good_ptr good;

	dim = *dimp;
	debug = *debugp;

	setDebugOption(state, debug);

	tree = sCreateTreeStr(merge, height, order, dim);
	good = sCreateGoodnessStr(pair_dist, dim);

	pairwiseDistance(tree, good, state);	
}

void pairwiseDistance(Tree_ptr tree, 
			Good_ptr good,
			Options state)
{
	int dim = good->dim_g;
	int root = dim-1;
	int left_num = 1;

	constructHeightOfLeaves(good, tree);
	traverseForAncestor(tree, good, root, left_num);
	if (debugOption(state))
	   printGoodnessStr(good);
}

void computePairwiseDistance(Good_ptr good,
				    Tree_ptr tree, 
				    int root, 
				    int left_num, 
				    int left_son_span, 
				    int right_son_span)
{
	int i,j;
	float ht_root;

 	/* get the height of root first */
	ht_root = getHeightValue(tree, root);

    	for (i=left_num; i<left_son_span; i++) 
	{
	    float ht_lson;
	    float ht_rson;
	    int l_elem = getOrderValue(tree,i); 
	
	    ht_lson = getHeightOfLeaf(good, l_elem );

            for (j=left_son_span; j < right_son_span; j++) 
	    {
		float goodness_value;
		int r_elem = getOrderValue(tree,j);
		ht_rson = getHeightOfLeaf(good, r_elem);

     		goodness_value = (2 * ht_root - ht_lson - ht_rson) / (float)2;
                if (goodness_value < 0)
		   goodness_value = -goodness_value;

		setDistBetweenLeaves(good, l_elem, r_elem,  goodness_value);
		/* set the lower triangular matrix also */
		setDistBetweenLeaves(good, r_elem, l_elem, goodness_value);         
            }
    	}
}
   

void constructHeightOfLeaves(Good_ptr good,
			     Tree_ptr tree)
{
	int row, col;
	float height;
	int merge_num;
	int dim_order = good->dim_g;
	int dim_merge = tree->size;

	for (row = 1; row <= dim_merge; row++)
	{
	    for (col = 1; col <= 2; col++)
	    {
		merge_num = getMergeValue(tree, row, col);
		if (merge_num < 0)	/* leaf node */
		{
		   height = getHeightValue(tree, row);
		   setHeightOfLeaf(good, -merge_num, height);
		}
	    }
	}

	/* consistency check */
	for (row = 1; row <= dim_order; row++)
	{
	    if (getHeightOfLeaf(good, row) == R_ABSURD)
	    {
		fprintf(stderr, "Not ALL leaves present for goodness
			computation...\n");
	    }
	}
}


void setHeightOfLeaf(Good_ptr good,
		     int pos,
		     float value)
{
	setRealVector(good->ht_arr, pos, value);
}

float getHeightOfLeaf(Good_ptr good,
		int pos)
{
	return getRealValue(good->ht_arr, pos);
}


void setDistBetweenLeaves(Good_ptr good,
			  int row,
			  int col,
			  float dist)
{
	setDistValue(good->sys_dist, row, col, dist);
}

float getDistBetweenLeaves(Good_ptr good,
			   int row,
			   int col)
{
	getDistValue(good->sys_dist, row, col);
}

void printGoodnessStr(Good_ptr good)
{
	I_vect dummy_suppress;
	dummy_suppress = createIntVector(good->dim_g);

	printf("PRINTING GOODNESS STRUCTURE NOW...\n");
	printDistMatrix(good->sys_dist, dummy_suppress);
	printRealVector(good->ht_arr, "Height array in Goodness structure");
}