#include <stdio.h>

#include "globals.h"
#include "bclust.h"
#include "binding.h"
#include "read_matrix.h"
#include "tree.h"
#include "goodness.h"


extern int dump;
extern char *suppress_list;

main(int argc, char **argv)
{
  
  Bind_mat bind;
  int dim;
  Options state;
  Tree_ptr tree;
  int i,j;
  int val;
  
  char *filename;
  
  setDebugOption(state, FALSE);
  setTraceOption(state, FALSE);
  setModeOption(state, 1);	/* default is expected binding */
  setClusterOption(state, 1);	/* default is many pairs in cluster */
  setReorderOption(state, 0);	/* default is do not reorder */

  filename = "";
  
  for (i =1; i < argc; i++)
    {
      if (*(argv[i]) == '-')
	{
	  switch (*(argv[i]+1))
	    {
	    case 'd': setDebugOption(state, TRUE);
	      break;
	    case 't': setTraceOption(state, TRUE);
	      break;
	    case 'm':
	      switch (*(argv[i]+2))
		{
		case 'e': setModeOption(state, 1);	/*expected*/
		  break;
		case 'r': setModeOption(state, 2);	/*recomputed*/
		  break;
		default: setModeOption(state, 1);	/*expected*/
		  break;
		} 
	      break;
	    case 'D': dump = atoi(argv[i]+2);
	      fprintf(stderr, "Dump binding matrix when %d clusters are found\n",
			dump);
	      break;
	    case 'S': suppress_list = argv[i]+2;
	      break;
	    case 'c':
	      switch (*(argv[i]+2))
		{
		case 's': setClusterOption(state, 1);	/*single*/
		  break;
		case 'm': setClusterOption(state, 2);	/*many*/
		  break;
		default: setClusterOption(state, 1);	/*many*/
		  break;
		} 
	      break;
	    case 'R':    setReorderOption(state, (*(argv[i]+2)));    /* Rorder */
	      break;

	    default: 
	      fprintf(stderr, "Unrecognized option \n");
	      break;
	    }
	} 
      else 
	{
	  filename = argv[i];
	}
    }
  
  printf("%s -- file being processed \n", filename);
  
  bind = fReadBindingMatrix(filename, &dim);
  
  tree = createTreeStr(dim);
  
  printf("calling bclust now....\n\n\n");
  bclust(dim, bind, tree, state);
  printTreeStr(tree);
  printf("bclust executed successfully....\n");
  /* convert height to levels */
  {
  	printf("CONVERTING HEIGHT TO LEVELS \n");
  	reconfigureLevels(tree);
  	if (debugOption(state))
	   printTreeStr(tree);
  }

  printf("GOODNESS COMPUTATION NOW:\n");
  {
	Good_ptr good = createGoodnessStr(dim);
	pairwiseDistance(tree, good, state);
	printGoodnessStr(good);
  }

  printf("Cluster analysis completed.......\n");
}

		 
	
