#include <stdio.h>

#include <strings.h>

#include "suppvect.h"
#include "globals.h"


void setSuppressVector(I_vect suppress, int member);


I_vect createSuppressVector(int dim)
{
	I_vect suppress;

	suppress = createIntVector(dim);
	return suppress;
}


void destroySuppressVector(I_vect suppress)
{
	free(suppress);
}


void updateSuppressVector(I_vect suppress,
			  Cl_vect clust_vect)
{
	int index;
	int dim = suppress->dim_i;

	for (index = 1; index <= dim; index++)
	{
	    if(isClustered(clust_vect, index) &&
		!isClustHead(clust_vect, index))
	    {
		setSuppressVector(suppress, index);
	    }
	}
}


void setSuppressVector(I_vect suppress, int member)
{
        /* Assertion */
        if (getIntValue(suppress, member) == TRUE) 
	{
          fprintf(stderr, "Suppressing element more than once \n");
        }
	setIntVector(suppress,member,TRUE);
}


void resetSuppressVector(I_vect suppress)
{
	resetIntVector(suppress, FALSE);
}


int isSuppressed(I_vect suppress, int member)
{
	return getIntValue(suppress, member);
}


int countSuppressed(I_vect suppress)
{
	int i;
	int dim = suppress->dim_i;
	int count = 0;

	for (i = 1; i <= dim; i++)
	{
	    if (isSuppressed(suppress, i))
		count++;
	}
         return count;
}



void printSuppressVector(I_vect suppress)
{
	int i;
	int val;
	int dim = suppress->dim_i;

	printf("Printing Suppress Vector\n");

	for (i = 1; i <= dim; i++)
	{
	    val = isSuppressed(suppress, i);
	    printf("%d ", val);
	}
	printf("\n");
}

void  stringSuppressVector(I_vect suppress_vector, char *suppress_list)
{
  	char *s1, *s2, *s3;
 	 char *index();
  
  	int hi, low, i;
  
  	while (strlen(suppress_list)>0) 
	{
    
    		s1 = index(suppress_list, ',');
    		s2 = index(suppress_list, '-');
    
    		if ((s1==NULL) && (s2==NULL))
		{
      		   low = hi = atoi(suppress_list);
      		   suppress_list = suppress_list + strlen(suppress_list);
    		} 
		else 
		{
      		   if ((s1 != NULL) && ((s2 ==NULL)  || (s1 < s2))) 
		   {
			/* comma found */
			*s1= '\0';
			low = hi = atoi(suppress_list);
			suppress_list = s1+1;
		   } 
		   else /* s1 > s2 */ 
		   {
		    	/* hyphen found */
		   	*s2 = '\0';
		   	low = atoi(suppress_list);
	
			s2++;
			s3 = index(s2, ',');
			if (s3 == NULL) 
			{
	  		   suppress_list = s2+strlen(s2);
			} 
			else 
			{
	  		   *s3 = '\0';
	  		   suppress_list = s3+1;
			}
			hi = atoi(s2);
      		   }
    		}
  
    		if (low == hi) 
      		   fprintf(stderr, "Suppressing column %d \n", low); 
		else
      		   fprintf(stderr, "Suppressing columns %d to %d \n", low, hi);

    		for (i=low; i <=hi; i++) 
      		    setSuppressVector(suppress_vector, i);
  	}
}

