#include <stdio.h>
#include "tree.h"
#include "errors.h"
#include "globals.h"

#define   matIndex(column_dim, row, col) (col-1) * (column_dim) + (row-1)

/* prototypes */

void updateMergeVector(Tree_ptr tree,
			int member1,
			int member2);

void updateHeightVector(Tree_ptr tree,
			float min_dist);

void setMergeVector(Tree_ptr tree,
		    int entry1,
		    int entry2);

void printMergeVector(Tree_ptr tree);


void setOrderVector(Tree_ptr tree,
			int value);

void printOrderVector(Tree_ptr tree);

void setHeightVector(Tree_ptr tree, int pos, float val);

void printHeightVector(Tree_ptr tree);

void setOldMergeToNew(Tree_ptr tree,
		 int old_value,
		 int new_value);

void shuffleMergeVector(Tree_ptr tree,
		    int entry1,
		    int entry2,
		    int pos);

void setMergePos(Tree_ptr tree, int pos, int val);

int mergedNode(Tree_ptr tree, int row);

int newLevel(Tree_ptr tree,
	     int root,
	     int node,
	     int curr_level);

void heightToLevelTraversal(Tree_ptr tree,
			    int root,
			    int level,
			    I_vect level_vect);

void copyLevelIntoTree(Tree_ptr tree,
		       I_vect level_vect);


/***********************now, the definitions ***************************/

Tree_ptr createTreeStr(int dim)
{
	Tree_ptr tree;

	tree = (Tree_ptr) emalloc(sizeof(Tree_str), "Tree structure");
	tree->order = (Order_vect) emalloc(sizeof(Order_str), "Order structure");

	tree->merge = (Merge_vect) ecalloc((dim-1)*2, sizeof(int), "Merge Vect");
	tree->height = (Height_vect) ecalloc(dim-1, sizeof(float), "Height Vect");
	tree->merge_pos = createIntVector(dim);

	tree->order->o_vect = (int *) ecalloc(dim, sizeof(int), "Order Vect");
	tree->order->order_pos = 0;
	tree->order->dim_o = dim;

	tree->size = dim - 1;
	tree->curr_pos = 0;

	return tree;
}

Tree_ptr sCreateTreeStr(Merge_vect merge,
			Height_vect height,
			int * order,
			int dim)
{
	Tree_ptr tree;

	tree = (Tree_ptr) emalloc(sizeof(Tree_str), "Tree Structure");
	tree->order = (Order_vect) emalloc(sizeof(Order_str), "Order structure");

	tree->merge_pos = createIntVector(dim);

	tree->merge = merge;
	tree->height = height;
	tree->order->o_vect = order;
	tree->curr_pos = 0;
	tree->order->order_pos = 0;
	tree->order->dim_o = dim;
	tree->size = dim - 1;

	return tree;
}

void destroyTreeStr(Tree_ptr tree)
{
	free(tree->merge);
	free(tree->height);
	free(tree->order->o_vect);
	free(tree->order);
	destroyIntVector(tree->merge_pos);

	free(tree);
}


void updateTreeStr(Tree_ptr tree,
			int member1,
			int member2,
			float min_dist)
{
	(tree->curr_pos)++;
	if (tree->curr_pos > tree->size)
	   myError("Tree's curr_pos going out of bounds...");

	updateMergeVector(tree,
			  member1,
			  member2);
	updateHeightVector(tree,
			   min_dist);
}


int getClustFromTree(Tree_ptr tree)
{
	return tree->curr_pos;
}


void printTreeStr(Tree_ptr tree)
{
	printMergeVector(tree);
	printOrderVector(tree);
	printHeightVector(tree);
}

void updateMergeVector(Tree_ptr tree,
			int member1,
			int member2)
				
{
	int merged_r, merged_c;
	I_vect merge_pos = tree->merge_pos;
	Merge_vect merge_vect = tree->merge;
	int dim = tree->size;
	int pos = tree->curr_pos;

	merged_r = mergedNode(tree, member1);
	merged_c = mergedNode(tree, member2);


	/* check if both have not been merged yet */

	if (!merged_r && !merged_c)
	{
	    /* pos has already been incremented */
	    setMergePos(tree, member1, pos);
	    setMergePos(tree, member2, pos);
	    setMergeVector(tree, -member1, -member2);	    
	}
	else if (merged_r && merged_c)
	{
	    setMergeVector(tree, merged_r, merged_c);
	    setMergePos(tree, member1, pos);
	    setMergePos(tree, member2, pos);
	}
	else	/* only one is merged previously */
	{
	    if (merged_r)
	    {
		setMergeVector(tree, merged_r, -member2);
	 	setMergePos(tree, member1, pos);
	 	setMergePos(tree, member2, pos);   
	    }
	    else
	    {
		setMergeVector(tree, merged_c, -member1);
		setMergePos(tree, member1, pos);
		setMergePos(tree, member2, pos);   
	
	    }
	}
}


/* update Merge vector BEFORE height vector */

void updateHeightVector(Tree_ptr tree,
			float min_dist)
{
	int first, second;
	float ht1, ht2;  	/* height of members */
	int mem_cl1, mem_cl2;	/* if the members are clusters */
	int pos = tree->curr_pos;

	ht1 = ht2 = 0;
	first = 1;
	second = 2;

	mem_cl1 = getMergeValue(tree, pos, first);
	mem_cl2 = getMergeValue(tree, pos, second);

	if (mem_cl1 > 0) 	/* positive if clust */
	   ht1 = getHeightValue(tree, mem_cl1);

	if (mem_cl2 > 0)
	   ht2 = getHeightValue(tree, mem_cl2);

	/* now take the heighest value of ht1, ht2, min_dist */

	if (ht1 > min_dist)
	   min_dist = ht1;
	if (ht2 > min_dist)
	   min_dist = ht2;

	/* store this value in height vector now */

	setHeightVector(tree, pos, min_dist);

}


int getMergeValue(Tree_ptr tree,
		  int row, int col)
{
	int dim = tree->size;
        return tree->merge[matIndex(dim, row, col)];
}


void setMergeVector(Tree_ptr tree,
		    int entry1,
		    int entry2)
{
	Merge_vect merge_vect = tree->merge;
	int dim = tree->size;
	int pos = tree->curr_pos;
	int temp;
	/****
	if (abs(entry2) < abs(entry1))
	{
	   temp = entry1;
	   entry1 = entry2;
	   entry2 = temp;
	}
	******/
        merge_vect[matIndex(dim, pos, 1)] = entry1;
        merge_vect[matIndex(dim, pos, 2)] = entry2;
}


void printMergeVector(Tree_ptr tree)
{
	int row, col, n_cols;
	int val;
	int dim = tree->size;

	n_cols = 2;

	printf("Printing Merge Vector...\n");

	/* only dim-1 rows are there */
	for (row = 1; row <= dim; row++)
	{
	    printf("%d\t", row);
	    for (col = 1; col <= n_cols; col++)
	    {
	  	val = getMergeValue(tree, row, col);
		printf("%4d", val);
	    }
	    printf("\n");
	}
}


int getOrderValue(Tree_ptr tree, int pos)
{
	return tree->order->o_vect[pos-1];
}

void setOrderVector(Tree_ptr tree, int val)
{
	tree->order->order_pos++;
	tree->order->o_vect[(tree->order->order_pos)-1] = val;
}

void printOrderVector(Tree_ptr tree)
{
	int i;
	int val;
	int dim = tree->order->dim_o;

	printf("Printing Order vector ....\n");
	for (i = 1; i <= dim; i++)
	{
	    val = getOrderValue(tree, i);
	    printf("%d ", val);
	}
	printf("\n");
}

float getHeightValue(Tree_ptr tree, int pos)
{
	return tree->height[pos-1];
}

void setHeightVector(Tree_ptr tree, int pos, float val)
{
	tree->height[pos-1] = val;
}


void printHeightVector(Tree_ptr tree)
{
	int i;
	float val;
	int dim = tree->size;

	printf("Printing Height vector...\n");
	for (i = 1; i <= dim; i++)
	{
	    val = getHeightValue(tree, i);
	    printf("%4.1f ", val);
	}
	printf("\n");
}


int mergedNode(Tree_ptr tree, int row)
{
	return getIntValue(tree->merge_pos, row);
}


void setMergePos(Tree_ptr tree, int pos, int val)
{
	setIntVector(tree->merge_pos, pos, val);
}


void makeOrderFromMerge(Tree_ptr tree)
{
	int i, j, value;
	int dim = tree->size;
	int o_dim = tree->order->dim_o;

	tree->order->order_pos = 0;

	for (i = 1; (i <=dim && tree->order->order_pos < o_dim); i++)
	{
	    for(j = 1; j <=2; j++)
	    {
		value = getMergeValue(tree, i, j);
		if (value < 0) 	/* leaf node */
		{
		   setOrderVector(tree, abs(value));
		}
	    }
	}
	/* consistency check */
	if (tree->order->order_pos < o_dim)
	   fprintf(stderr, "Not all elements are clustered.....\n"); 
}


static  void traverseOrderVector(Tree_ptr tree,
                		int root)
{

   	if (isleaf(tree, root)) 
	{
	   setOrderVector(tree, -root);
   	} 
	else 
	{
           traverseOrderVector(tree, leftnode(tree, root));
           traverseOrderVector(tree, rightnode(tree, root));
   	}
}


void makeOrderVector(Tree_ptr tree)
{
	tree->order->order_pos = 0;
	traverseOrderVector(tree, tree->size);
}


void reconfigureLevels(Tree_ptr tree) 
{
	int root = tree->size;		/* root as in merge_vector */
	I_vect level_vect = createIntVector(tree->size);
	/* level_vect stores the computed levels, assume level(root) = 0 */
	int level =0;

	heightToLevelTraversal(tree, root, level, level_vect);
	copyLevelIntoTree(tree, level_vect);
}

void heightToLevelTraversal(Tree_ptr tree,
			    int root,
			    int level,
			    I_vect level_vect)
{
	if (!isleaf(tree, root))
	{ 
	   int left_level, right_level;

	   setIntVector(level_vect, root, level);
	   left_level = newLevel(tree,
				 root,
				 leftnode(tree, root),
				 level);
	   right_level = newLevel(tree,
				  root,
				  rightnode(tree, root),
				  level);
	   heightToLevelTraversal(tree, 
				  leftnode(tree,root),
				  left_level,
				  level_vect);
	   heightToLevelTraversal(tree, 
				  rightnode(tree,root),
				  right_level,
				  level_vect);

	}
}

int newLevel(Tree_ptr tree,
	     int root,
	     int node,
	     int curr_level)
{
	if (!isleaf(tree, node))
	{
	   if (getHeightValue(tree, node) <
	       getHeightValue(tree, root))
	       curr_level++;
	}
	return curr_level;
}

void copyLevelIntoTree(Tree_ptr tree,
		       I_vect level_vect)
{
	int dim = tree->size;
	int i;
	float val;
	int max = 0;
	
	/* consistency check */
	if (dim != level_vect->dim_i)
	   fprintf(stderr, "Level_vect of incorrect size\n");
	
	/* find the maximum depth in order to compute actual levels */
	for (i = 1; i <= dim; i++)
	{
	    val = getIntValue(level_vect, i);
	    if ( val > max) 
		max = val;  	    
	}
	max = (float) max;		/* for further operations */
	for (i =1; i <= dim; i++)
	{
	    val = max - (float) getIntValue(level_vect, i); /* defined levels */
	    setHeightVector(tree, i, val);
	}
}


void sReconfigureLevels(Merge_vect merge_arr,
			Height_vect ht_arr,
			int * order_arr,
			int *dimp)
{
	Tree_ptr tree = sCreateTreeStr(merge_arr,
					ht_arr,
					order_arr,
					*dimp);

	reconfigureLevels(tree);	
}
