
#    Interface to C functions ...

# To compute pairwise distance

CpairwiseDistance <- function (h, debug=0, trace=0)
{
   merge <- h$merge
   height <- h$height
   order <- h$order

   ndim <- length(order)

   pdspace <- matrix(0, nrow=ndim, ncol=ndim)

   storage.mode(merge) <- "integer"
   storage.mode(height) <- "single"
   storage.mode(order) <- "integer"
   storage.mode(ndim) <- "integer"
   storage.mode(pdspace) <- "single"

   COutput <- .C( "S_pairwiseDistance",
	 as.integer(ndim),
         as.matrix(merge),
         as.matrix(height),
         as.matrix(order),
	 pd = as.matrix(pdspace),
         as.integer(debug)
        )

    COutput$pd
}

# To compute distance

Cdist <- function(mat, mode=1, debug=0, trace=0)
{
    ndim <- nrow(mat)
    dspace <- matrix(0, nrow=ndim, ncol=ndim)

    storage.mode(mat) <- "integer"
    storage.mode(dspace) <- "single"
    storage.mode(ndim) <- "integer"

    COutput <- .C("S_computeDistance",
             as.integer(ndim),
             as.matrix(mat),
             as.integer(mode),
             d = as.matrix(dspace),
             as.integer(debug)
          )

    COutput$d
}

# To perform clustering

Cbclust <- function (
    bind,
    debug=0,
    trace=0,
    mode=1,
    cluster=1,
    reorder="t"
   ) {

   ndim <- nrow(bind)
   merge <- matrix(1:((ndim-1)*2), nrow=(ndim-1), ncol=2)

   storage.mode(bind) <- "integer"
   storage.mode(merge) <- "integer"

   h <- .C(  "S_bclust",
	    as.integer(nrow(bind)),
	    as.matrix(bind),
	    merge = as.matrix(merge),
	    height = as.matrix(single(ndim-1)),
	    order = as.matrix(integer(ndim)),
	    as.integer(debug),
	    as.integer(mode),
	    as.integer(trace),
	    as.integer(cluster),
	    as.character(reorder),

       )

   list(merge=h$merge, height=c(h$height), order=c(h$order))

}
