

do.cor.test <- function(
	 database=clustsys.data,
         method1= "e",
         method2= "r",
         method= "spearman",
         querytype= "all",
         variable= "goodness",
         ...
     )

{
    var.data <-  extract.across(database=database, variable=variable)[[querytype]]
    x <- var.data[,method1]
    y <- var.data[,method2]
    result <- cor.test(x, y, method=method, ...)
    result[["data.name"]] <- paste("Querytype =", querytype,
                                   "Variable =",  variable,
                                   "x = ", method1,
                                   "y = ", method2)
    result
}

do.t.test <- function(
	 database=clustsys.data,
         method1= "e",
         method2= "r",
         querytype= "all",
         variable= "goodness",
         alternative="greater",
         mu=0,
         ...
     )

{
    var.data <-  extract.across(database=database, variable=variable)[[querytype]]
    x <- var.data[,method1]
    y <- var.data[,method2]
    result <- t.test(x, y, paired=T, mu=mu, alternative=alternative)
    result[["data.name"]] <- paste("Querytype =", querytype,
                                   "Variable =",  variable,
                                   "x = ", method1,
                                   "y = ", method2)
    result
}

do.wilcox.test <- function(
	 database=clustsys.data,
         method1= "e",
         method2= "r",
         querytype= "all",
         variable= "goodness",
         ...
     )

{
    var.data <-  extract.across(database=database, variable=variable)[[querytype]]
    x <- var.data[,method1]
    y <- var.data[,method2]
    result <- wilcox.test(x, y, paired=T)
    result[["data.name"]] <- paste("Querytype =", querytype,
                                   "Variable =",  variable,
                                   "x = ", method1,
                                   "y = ", method2)
    result
}
do.var.test <- function(
	 database=clustsys.data,
         method1= "e",
         method2= "r",
         querytype= "all",
         variable= "goodness",
         ...
     )

{
    var.data <-  extract.across(database=database, variable=variable)[[querytype]]
    x <- var.data[,method1]
    y <- var.data[,method2]
    result <- var.test(x, y)
    result[["data.name"]] <- paste("Querytype =", querytype,
                                   "Variable =",  variable,
                                   "x = ", method1,
                                   "y = ", method2)
    result
}

clustsys.apply <- function(database=clustsys.data,
                           query="all",
                           ...) {
     h <-  lapply(database[[query]], ...)
     h$year <- database$year
     h
}


variables.mean <- function (database=clustsys.data,
                          query="all") {

     h <-  clustsys.apply(database=database, query=query, apply, 2, mean)
     h$function.ratio <- apply(function.ratios(database=database, query=query), 2, mean)
     h
}

test.shape <- function(
	 database=clustsys.data,
         method= "e",
         querytype= "all",
         variable= "goodness",
         ...
     )

{
    par.save <- par()
    var.data <-  extract.across(database=database, variable=variable)[[querytype]]
    x <- var.data[,method]
    eda.shape(x, heading=paste(database$year, database$methods.print.name[[method]], database$variables.print.name[[variable]]))

     par(par.save)
  NULL
}

test.ts <- function(
	 database=clustsys.data,
         method= "e",
         querytype= "all",
         variable= "goodness",
         ...
     )

{
    par.save <- par()
    var.data <-  extract.across(database=database, variable=variable)[[querytype]]
    x <- var.data[,method]

    eda.ts(x)

     par(par.save)
   NULL
}

# Function for exploratory data analysis (EDA). Taken from the S-Plus
# User's Manual, page 14-7.

eda.shape <- function(x, heading="") {


     par(mfrow=c(2,2))

     hist(x)
     boxplot(x)
     iqd <- summary(x)[5] - summary(x)[2]
     plot(density(x,width=2*iqd), xlab="x",
          ylab="", type="l")

     qqnorm(x)
     qqline(x)
     title(mean=heading, cex=1.5)

     NULL
}

eda.ts <- function(x) {
    par(mfrow=c(2,1))
    tsplot(x)
    acf(x)
    invisible()
}
