
clustsys <-
     function(matrixfile,
	labelsfile="",
	factorfile="",
        fmatchfile="",
	method="hb",
	displaytype="motif",
	distancefile="",
        fileprefix="",
        system="",
        querytype="",
        key="",
        control="",
	debug=0,
	trace=0,
        interface="demo"
     )
{

   if (interface == "batch") {
     sysparam <- makeSysParam(system=system, querytype=querytype, key=key, method=method, fileprefix=fileprefix)

     if (!existsKeyEntry(sysparam)) {
         print(paste("The Key", sysparam$key, " is not defined in the database"))
         abort("You probably forgot to use the -k key option")
     }
   }
     # read matrix file
     mat <- getMatrix(matrixfile)
     lb <<- getLabels(labelsfile, nrow(mat))

     if (nrow(mat)==0) {
        abort("The binding matrix is empty. Cannot proceed")
     }

     # Validate data dimensions

     validate.dimensions(nrow(mat), length(lb), "1")

     # perform clustering
     h <<- doClustSys(mat, method, debug, trace)

     # output dendrogram and  record it
     outputH(h, lb, sysparam, displaytype)

     if (interface == "demo") return(h)

     record.h(value=h, sysparam=sysparam)

     # stop if -E specified
     if (control == "E") return(h)

     # pairwise distance 
     pd <- CpairwiseDistance(h, debug)

     # stop if -c specified
     if (control == "c") {
       if (interface=="batch") print(pd)
       return(pd)
     }

     if (fmatchfile == "" ) return(h)

     #  Compute goodness
     fmatch <- getMatrix(fmatchfile)

     # Validate dimensions of matrix
     validate.dimensions(length(lb), nrow(fmatch), "2")

     # check if range is non zero... otherwise the goodness is undefined
     if (min(h$height) == max(h$height)) {
         print("WARNING:... All the functions are clustered at the same level")
         print("You will find the goodness measures undefined")
     }

#     g <- systemGoodness(min(h$height), max(h$height), pd, fmatch, debug)

# ********************************************************
# calculate the 1 - delta part of the congruence measure
     g <- 1 - delta(fmatch, pd)
# ********************************************************

     outputG(g, sysparam, displaytype)

     record.variables(goodness=g, sysparam=sysparam)

     print("a  4")
     if (interface=="batch") {
	print("System goodness")
	print(g)
     }

    if (control == "S")  return(g)

    if (factorfile == "") return(g)

    # compute file-factor
    f <- getFactorInfo(factorfile, debug)

    outputF(f, sysparam, displaytype)
    record.variables(functions.in.system=f$functions.in.system, sysparam=sysparam)
    record.variables(functions.used=f$functions.used, sysparam=sysparam)

    if (interface== "batch") {
        print("File factor")
        print(f)
    }

   # factored goodness
#   fg <- (f$functions.used/f$functions.in.system)*g

# ********************************************************
# calculate the whole congruence measure
   fg <- intersectionRatio(f$functions.used, f$functions.in.system) * g
# ********************************************************

   if (interface == "batch") {
       print("File factor * system goodness = ")
       print(fg)
   }

   outputFG(fg, sysparam, displaytype)
   record.variables(system.goodness=fg, sysparam=sysparam)
   return(0)
}

getMatrix <- function (matrixfile)
{
  mat <- scan(matrixfile)
  dim <- sqrt(length(mat))
  matrix(mat, nrow=dim)
}

getLabels <- function (labelsfile, ndim) 
{
     if (labelsfile != "") {
         scan(labelsfile, what="")
     } else {
         1:ndim
     }
}

getFactorInfo <- function (factorfile, debug=0)
{
    input <- scan(factorfile)

    # first number is: all functions
    # second number is: number of functions clustered

    if (debug != 0) {
       print("Factor file info")
       print("Total functions, functions clustered")
       print(c(input[1], input[2]))
    } 
    list(functions.in.system=input[1], functions.used = input[2])
}


doClustSys <- function (mat, method="e", ...)
{
  # method codes
  #       e -- expected binding 
  #       r -- recomputed binding
  #       he -- hclust/expected binding
  #       hr -- hclust/recomputed binding
  #       hb -- hclust/matrix binding


    if (method == "e") {
          Cbclust(mat, mode=1, ...)
    } else if (method == "r") {
          Cbclust(mat, mode=2, ...) 
    } else if (method == "he") { 
          hclust(Cdist(mat, mode=1, ...))
    } else if (method == "hr") {
          hclust(Cdist(mat, mode=2, ...))
    } else if (method == "hb") {
          hclust(sim=mat)
    } else {
        print(paste("Unrecognized clustering option", method))
    }
}

makeSysParam <- function(...)
{
   list(...)
}

validate.dimensions <- 
       function(dim1, dim2, position)
{ 
   if (dim1!=dim2) print(paste("Dimension mismatch at checkpoint: ", position))
}

demo.clustsys <- 
     function(matrixfile,
	labelsfile="",
	method="he",
        debug= 0,
        trace= 0
     )
{

     # read matrix file
     mat <- getMatrix(matrixfile)
     lb <<- getLabels(labelsfile, nrow(mat))

     if (nrow(mat)==0) {
        abort("The binding matrix is empty. Cannot proceed")
     }

     # Validate data dimensions
     validate.dimensions(nrow(mat), length(lb), "1")

     # perform clustering
     h <- doClustSys(mat, method, debug, trace)
     plclust(h, labels=lb)
     h
}
