

# Transfer the fields of exp.param to clustsys.data
# of various years

transfer.92  <- function () {
    set.for.92();
    for (n in names(exp.param)) {
      clustsys92.data[[n]] <<- exp.param[[n]]
    }

}


transfer.93  <- function () {
    set.for.93();
    for (n in names(exp.param)) {
      clustsys93.data[[n]] <<- exp.param[[n]]
    }
}


transfer.94  <- function () {
    set.for.94();
    for (n in names(exp.param)) {
      clustsys94.data[[n]] <<- exp.param[[n]]
    }

}


transfer.9X  <- function () {
    set.for.9X();
    for (n in names(exp.param)) {
      clustsys9X.data[[n]] <<- exp.param[[n]]
    }

}


multiple.mean <- function (variable="goodness") {

 v92 <-   variables.mean(clustsys92.data)[[variable]]
 v93 <-   variables.mean(clustsys93.data)[[variable]]
 v94 <-   variables.mean(clustsys94NA.data)[[variable]]
 v9X <-   variables.mean(clustsys9XNA.data)[[variable]]

 year.names    <- c("92", "93", "94", "9X")
 method.names  <- names(v92)

 m <-  matrix(c(v92, v93, v94, v9X), 
              byrow=T,
              nrow=length(year.names),
              ncol=length(method.names),
              dimnames=list(year.names,method.names))

 m
}

plot.multiple.mean  <- function(
       methods=exp.param[["clustering.methods"]],
       variable = "goodness",
       query="all",
       type="b",
       xtck=T,
       cex=1.5,
       x.legend=1,
       y.legend=0.9,
       mar=c(5,5.5,0,1)+0.6,
       select.years=c("92", "93", "94"), 
       ...
      )
{

   par.save <- par()

  # default database to use to get various parameter values

    database <- clustsys.data

  # get the multiple means
     m <- multiple.mean(variable=variable)
     m.ts <- ts(m, start=1992, names=dimnames(m)[[2]])

  # The actual matrix to be plotted
     v <- m

  # get x and y label messages
     x.label <- "Years"
     y.label <- paste("Mean ", database[["variables.print.name"]][variable])

  # Change the margins to accomodate the labels

   if (cex > 1) {
      par(mar=mar)
   }

  # get the minimum and maximum of this plot .. to draw the vertical tick line
    y.min <- min(v[select.years,methods])
    y.max <- max(v[select.years,methods])

   # Define the plot region and axes
   dummy.data <- c(y.min, y.max, rep(y.min, length(select.years)-2))
   dummy.ts   <- ts(dummy.data, start=1992)
   tsplot(dummy.data, type="n", lty=1, pch=1,
           ylab= "", xlab="",  ...)
   title(main="", ylab= y.label, xlab=x.label, cex=cex)


   # draw vertical tick line if requested
   if (xtck) {
       x1 <- 1:length(select.years)
       y1 <- rep(y.min+0.01, length(select.years))
       y2 <- rep(y.max, length(select.years))
       segments(x1, y1, x1, y2, lty=2, xaxt="d", yaxt="d", lwd=0.1)
   }

  # number of methods for which the variable is plotted
    methods.num <- length(methods)

   # draw the legend box. The placement was found by trial and error
   # legend should be drawn after the vertical ticks
   # So that it overwrites the overlapping region...

  # names of methods to be printed in legend
     legend.text <- database[["methods.print.name"]][methods]

     legend(x.legend, y.legend, legend.text,
            lty=lty.set[1:methods.num],
            cex=1)

  # method number in methods list
    i <- 1

  # now plot the variables for the other methods

    par(yaxs="d", xaxs="d")
     for (method in methods) {
        par(new=T)
        plot(v[select.years,method], type=type, pch=i, lty=lty.set[i], xlab="", ylab="", axes=F, ...)
        i <- i+1
    }

  # restore status of all the par variables
   par(par.save)
   par(new=F)

  NULL
}


plot.multiple.mean.2  <- function(
       database=clustsys.data,
       variable = "goodness",
       query="all",
       type="b",
       xtck=T,
       cex=1.5,
       start.year=1992,
       x.legend=start.year+1,
       y.legend=0.9,
       mar=c(5,5.5,0,1)+0.6,
       select.years=c("92", "93", "94"), 
       ...
      )
{

   par.save <- par()

   methods <- database$clustering.methods

  # get the multiple means
     m <- multiple.mean(variable=variable)[select.years,]
     m.ts <- ts(m, start=start.year, names=dimnames(m)[[2]])

  # get x and y label messages
     x.label <- "Years"
     y.label <- paste("Mean ", database[["variables.print.name"]][variable])

  # Change the margins to accomodate the labels

   if (cex > 1) {
      par(mar=mar)
   }

  # get the minimum and maximum of this plot .. to draw the vertical tick line
    y.min <- min(m)
    y.max <- max(m)

   # Define the plot region and axes
   tsplot(m.ts, type="n", lty=1, pch=1,
           ylab= "", xlab="",  ...)
   title(main="", ylab= y.label, xlab=x.label, cex=cex)


   # draw vertical tick line if requested
   if (xtck) {
       x1 <- (1:length(select.years))+start.year-1
       y1 <- rep(y.min+0.01, length(select.years))
       y2 <- rep(y.max, length(select.years))
       segments(x1, y1, x1, y2, lty=2, xaxt="d", yaxt="d", lwd=0.1)
   }

  # number of methods for which the variable is plotted
    methods.num <- length(methods)

   # draw the legend box. The placement was found by trial and error
   # legend should be drawn after the vertical ticks
   # So that it overwrites the overlapping region...

  # names of methods to be printed in legend
     legend.text <- database[["methods.print.name"]][methods]

     legend(x.legend, y.legend, legend.text, 
            lty=lty.set[1:methods.num], 
            cex=0.85*cex)


  # method number in methods list
    i <- 1

  # now plot the variables for the other methods


    par(yaxs="d", xaxs="d")
     for (method in methods) {
        par(new=T)
        tslines(ts(m.ts[,method], start=start.year),
                type=type,
                pch=i,
                lty=lty.set[i],
                xlab="", ylab="",
                axes=F,
                yaxs="d",
                xaxs="d",...)
        i <- i+1
    }
  # restore status of all the par variables
   par(par.save)
   par(new=F)

  NULL
}

