
#########  EXPERIMENT 2 ###########

exp.2.tree.good <- function (...)  
{
       h.tree(h15, lb15, "Good", cex=cex.tree, mar=mar.tree, mex=mex.tree, ...)
}

exp.2.tree.fair <- function (...)  
{
      h.tree(h14, lb14, "Fair", cex=cex.tree, mar=mar.tree, mex=mex.tree, ...)
}

exp.2.tree.poor <- function (...)  
{
      h.tree(h29, lb29, "Poor", cex=cex.tree, mar=mar.tree, mex=mex.tree, ...)

}

exp.2.hist.a <- function(
            database= clustsys.data,
            ...

) {
   hist.variable(database=database,
            querytype="fields",
            variable="goodness",
            method="e",
           ...)
}

exp.2.hist.b <- function(
            database= clustsys.data,
            ...

) {

   hist.variable(database=database,
            querytype="fields",
            variable="system.goodness",
            method="e",
           ...)
}

#########  EXPERIMENT 3 ###########


exp.3.hist.a <- function(
            database= clustsys.data,
            ...
) {

   hist.variable(database=database,
            querytype="all",
            variable="system.goodness",
            method="e",
           ...)
}


exp.3.hist.b <- function(
            database= clustsys.data,
            ...
) {

   hist.variable(database=database,
            querytype="all",
            variable="system.goodness",
            method="r",
           ...)
}


exp.3.plot <- function (
       database=clustsys.data,
       methods=c("e", "r"),
       ...
) {
     plot.variable(database=database, methods=methods, x.legend=12, y.legend=0.48, cex=cex.plot, ...)
}

#########  EXPERIMENT 4 ###########


exp.4.hist.a <- function(
            database= clustsys.data,
            ...
) {

   hist.variable(database=database,
            querytype="all",
            variable="system.goodness",
            method="hb",
           ...)
}

exp.4.hist.b <- function(
            database= clustsys.data,
            ...
) {

   hist.variable(database=database,
            querytype="all",
            variable="system.goodness",
            method="he",
           ...)
}
exp.4.hist.c <- function(
            database= clustsys.data,
            ...
) {

   hist.variable(database=database,
            querytype="all",
            variable="system.goodness",
            method="hr",
           ...)
}

exp.4.plot <- function (
       database=database,
       methods=c("he", "hr", "hb"),
       ...
) {
     plot.variable(database=database,
                   methods=methods, 
                   x.legend=c(12,19),
                   y.legend=c(0.50, 0.455),
                   cex=cex.plot,
                    ...)
}


sample.tree <- c(
     "exp.2.tree.good",
     "exp.2.tree.fair",
     "exp.2.tree.poor"
    )

sample.hist <- c(
#     "exp.2.hist.a",
#     "exp.2.hist.b",
     "exp.3.hist.a",
     "exp.3.hist.b",
     "exp.4.hist.a",
     "exp.4.hist.b",
     "exp.4.hist.c"
   )

sample.plot<- c(
     "exp.3.plot",
     "exp.4.plot")

exec.and.dump.screen <-  function (
     funnames=c(sample.tree, sample.plot, sample.hist),
     dir= output.dir,
     width=6,
     height=6*0.75,
     ...)
{
      for (i in funnames) {
        print(i)
        i.call <- call(i)
        eval(i.call)
        i.epsfile <- paste(dir, "/", i, ".eps", sep="")
        dump.screen(i.epsfile, width=width, height=height, ...)
      }
}


dump.tree <- function (...) {
     exec.and.dump.screen(funnames= sample.tree,
                 dir= output.dir,
                 width=width.tree,
                 height=height.tree,
                 ...
                )
}

dump.plot <- function (...) {
     exec.and.dump.screen(funnames= sample.plot,
                 dir= output.dir,
                 width=width.plot,
                 height=height.plot,
                 ...
                )
}

dump.exp.hist <- function (funnames= sample.hist, ...) {
     exec.and.dump.screen(funnames= funnames,
                 dir= output.dir,
                 width=width.hist,
                 height=height.hist,
                 ...
                )
}

dump.all <- function () {
     dump.tree()
     dump.plot()
     dump.exp.hist()
}


######### Computation for publisher ########

paper.height <- 11
paper.width  <- 8.5

pub.param <- function (
        fig.width,
        fig.height,
        p.height = paper.height,
        p.width = paper.width
    ) 
{
     v.offset <- (p.height - fig.height)/2
     h.offset <- (p.width - fig.width)/2

     list(height=fig.height, width=fig.width, h.offset=h.offset, v.offset=v.offset)
}

compute.pub.param <- function (...)
{
    print("For trees")
    print(pub.param(fig.height=height.tree, fig.width=width.tree))

    print("For histograms")
    print(pub.param(fig.height=height.hist, fig.width=width.hist))

    print("For plots")
    print(pub.param(fig.height=height.plot, fig.width=width.plot))

  NULL
}

####### ########### Dump all histograms ###################

dump.all.hists <- function(database=clustsys.data, ...) {

    for (query in valid.queries(database)) {

     # functions used
      hist.variable(database=database,
                    querytype=query,
                    variable="functions.used", ...)
     # functions in system
      hist.variable(database=database,
                    querytype=query,
                    variable="functions.in.system", ...)

     # function ratio
      hist.function.ratios(database=database,
		    querytype=query,
                    ...)

     # system goodness and goodness for all methods

     for (v in c("goodness", "system.goodness")) {
	 for (m in database$clustering.methods) {
	  hist.variable(database=database,
			querytype=query,
			method=m,
			variable=v, ...)
	}
     }
   }
}


################################################
##
##    statistical analysis for the paper
##
###############################################

dump.analysis <- function (file="paired-t.analysis", database=clustsys9XNA.data) {

    output.file <- paste(output.dir, file, sep="/")

    print(paste("Putting analysis results in: ", output.file))
    sink(output.file)

    # Methods in decreasing order of their mean of "goodness"
    method.list <- c("he", "e", "r", "hr", "hb")

    for (method1 in 1:(length(method.list)-1)) {
       for (method2 in (method1+1):length(method.list)) {
         h <- do.t.test(alternative="greater", 
                   method1=method.list[method1], 
                   method2=method.list[method2],
                  database=database)
         print(h)
       }
    }


    sink()

}

dump.means  <- function(file="variable-means.analysis") {
    output.file <- paste(output.dir, file, sep="/")

     v92 <-   variables.mean(clustsys92.data)
     v93 <-   variables.mean(clustsys93.data)
     v94 <-   variables.mean(clustsys94NA.data)
     v9X <-   variables.mean(clustsys9XNA.data)


    print(paste("Putting analysis results in: ", output.file))
    sink(output.file)

    print("================ 92  ===============")
    print(v92)
    print("================ 93  ===============")
    print(v93)
    print("================ 94  ===============")
    print(v94)
    print("================ 9X  ===============")
    print(v9X)

    sink()

}
