
######## Parameters for the experiment ############

##### The variable clustsys.data is used to keep all the experiment information
#####

exp.param <- list(

             # clustering methods used

                 clustering.methods = c("e", "r", "he", "hr", "hb"),

              # print name for methods
                 methods.print.name = c("Expected", 
                                        "Recomputed",
                                        "S/expected",
                                        "S/recomputed",
                                        "S/binding"),

             # Keys of all the systems clustered

                 systems.clustered  = c(paste("p92", c(1:9), sep=".0"),
                                        paste("p92", c(10:20), sep=".")),
                 system.abbr        = paste("", c(1:20)),
 
             # Various types of queries for which data exists

                 querytype          =  c("fields",          "all"),

             # Variables whose information is collected

		 variables          = c("functions.used", 
					"functions.in.system",
					"goodness",
					"system.goodness"),
		 variables.print.name = c("Functions used",
					"Functions in system",
					"System goodness",
					"Factored goodness"),

		 select.programs = c()
         )


names(exp.param[["methods.print.name"]]) <- exp.param[["clustering.methods"]]
names(exp.param[["variables.print.name"]]) <- exp.param[["variables"]]

exp.param$select.programs <- 1:(length(exp.param$systems.clustered))

gen.pgm.names <- function(year, last.pgm) {
 if (last.pgm > 9) {
  pgm.range <-    c(paste("0", 1:9, sep=""),
                    paste(10:last.pgm, sep=""))
 } else {
  pgm.range <-   paste("0", 1:last.pgm, sep="")
 }
  paste("p", year, ".",  pgm.range, sep="")
}

pgm.names92 <- gen.pgm.names(92, 20)
pgm.names93 <- gen.pgm.names(93, 22)
pgm.names94 <- gen.pgm.names(94, 24)
pgm.names9X <- c(pgm.names92, pgm.names93, pgm.names94)


set.system.for.year <- function (year, pgm.names) {
  exp.param$year <<- paste(year)
  exp.param$systems.clustered  <<- pgm.names

  exp.param$system.abbr        <<- paste(1:length(pgm.names))
  exp.param$select.programs <<- 1:length(pgm.names)

  exp.param
}


set.for.92 <- function () {
   set.system.for.year(92, pgm.names92)
}

set.for.93 <- function () {
   set.system.for.year(93, pgm.names93)
}

set.for.94 <- function () {
   set.system.for.year(94, pgm.names94)
}

set.for.9X <- function () {
  set.system.for.year("9X", pgm.names9X)
}

init.clustsys.data <- function () {
	clustsys92.data <<- set.system.92()
	clustsys93.data <<- set.system.93()
	clustsys94.data <<- set.system.94()
	clustsys9X.data <<- set.system.9X()
}

