hist.variable <- function(
            database  = clustsys.data,
            querytype = "all",
            variable  = "system.goodness",
            method    = "e",
            cex=cex.hist,
            mex=mex.hist,
            mar=mar.hist,
            dump=T,
            ...

) {

   d.p <-  database$select.programs

   var.print.name    <- database$variables.print.name[[variable]]
   method.print.name <- database$methods.print.name[[method]]

   data <- database[[querytype]][[variable]][d.p,method]

 
   h <- hist.percent(data, 
        xlab = var.print.name,
        cex=cex,
        mar=mar,
        mex=mex,
        heading= paste("Method:", method.print.name, "Query:", querytype, sep=" "),
        ...)
     hist.annotate(database) 

   if (dump) dump.hist(database=database, querytype=querytype, method=method, variable=variable, ...)
   h
}


hist.annotate  <- function(database, cex=1.2, side=4) {
    mtext(side=side, paste("Year", database$year), cex=cex)
}


hist.function.ratios <- function(
         database=clustsys.data,
         querytype="all",
	 heading="Function ratios",
	 xlab="Ratio of functions used",
	 cex=cex.hist,
	 mex=mex.hist,
	 mar=mar.hist, 
         dump=T,
         ...) {

      f.ratios <- function.ratios(database=database, querytype=querytype)

     h <- hist.percent(f.ratios,
	           heading=paste("Query: ", querytype, sep=" "),
                   xlab=xlab,
                   cex=cex,
                   mex=mex,
                   mar=mar, ...)

     hist.annotate(database)
      
   if (dump) dump.hist(database=database, querytype=querytype, method="", variable="function.ratios", ...)
     h
}

hist.percent <- function(
       data, 
       cex=1.5,
       mex=0.5,
       mar= c(2,2,2,1)+0.1,
       raise=0.6,
       heading="",
       ...
) {
   par.save <- par()
 
  # change margins
     par(mar=mar)

  # draw histogram
     h <- hist(data, cex=cex, plot=F, mar=mar, ylim=c(0,10),   ...)

  # put % on top of bars
     y  <- h$counts
     percent <- format(round(h$counts *100/sum(h$counts),1))
     x <- hist(data, cex=cex, ...)
     text(x, y+raise, paste(percent, "%", sep=""), cex=cex)

     h$x <- x
     h$y <- y

   hist.legend(data, h, cex=cex, mex=mex, mar=mar, ...)

   title(heading, cex=cex+mex)

   # restore screen parameters
   par(par.save)

   h
}

hist.legend <- function (
       data, 
       histogram,
       cex=1.5,
       mex=0.5,
       mar= c(2,2,2,1)+0.1,
       x.legend=0,
       y.legend=1,
       ...
) {
   par.save <- par()

     x.range <- range(histogram$breaks) 
     y.range <- range(histogram$y)

     bar.width <- diff(x.range)/length(histogram$breaks)


    ## x.legend and y.legend to place box on the RIGHT top 
#     x.legend <- c(x.range[2] - diff(x.range)/4, x.range[2])
#     y.legend <- c(y.range[2] - diff(y.range)/4, y.range[2])

     ## x.legend and y.legend to place box on the LEFT top 
     x.legend <- c(x.range[1], x.range[1] + diff(x.range)/4) - (bar.width/4)
     y.legend <- c(y.range[2] + diff(y.range)/4, y.range[2])


   # Put mean and std. deviation

   # A comment on the use of format below. If all was well, format
   # would be redundant. But there seems to be some problem when paste
   # converts std.dev to character. In one situation, it loses the
   # rounding done by round. Using format seems to fix it.

     mean.legend <- paste("Mean:", format(round(mean(data), 2)))
     std.legend <-  paste("S. dev:", format(round(sqrt(var(data)), 2)))

     legend(x.legend,
            y.legend,
            c(mean.legend, std.legend),
            cex=cex,
            bty="n", 
            ...)

   # restore screen parameters
   par(par.save)
}


plot.pairs <- function(
       database= clustsys.data,
       variable = "system.goodness",
       query="all",
       methods=exp.param[["clustering.methods"]],
       type="b",
       cex=1.2,
       ...
      )
{

  # The actual matrix to be plotted
     v <- extract.across(variable=variable, database=database)[[query]]

  # number of programs for which information is plotted
    num.programs <- nrow(v)

  # d.p is programs to be displayed. Only programs stated
  # in the select.programs are displayed. 
    d.p <- database$select.programs

  # names of methods to be printed in the boxes
     method.names <- database$methods.print.name[methods]


     pairs(v[d.p,methods,drop=F], labels=method.names, panel=function(x,y){ points(x,y); lines(lowess(x,y))}, ...)
}


h.tree <- function(
     htree,
     label,
     message,
     cex=1.5,
     lwd=1.9,
     mex=0.5,
     mar=c(1,3,3,0),
     ...)
{
   save.mar <- par("mar")
   par(mar=mar)
   plclust(htree, label=label, cex=cex, yaxt=par()$yaxt, lwd=lwd, mar=mar, ...)
   title(message, cex=(cex+mex))
   par(mar=save.mar)
}

##########################################################################
##
##     The following functions do not provide any significant
##     information for clustsys related analysis
##
##########################################################################


barchart <- 
   function(querytype="all",
            variable="goodness",
            database= clustsys.data
           ) {
   methods <- dimnames(database[[querytype]][[variable]])[[2]]
   barplot(database[[querytype]][[variable]], names=methods)
   title(paste("Query=", querytype, "Var= ", variable))
}


barchart.row <- 
   function(querytype="all",
            variable="goodness",
            database= clustsys.data,
            row="p94.01"
           ) {
   methods <- dimnames(database[[querytype]][[variable]])[[2]]
   barplot(database[[querytype]][[variable]][row,,drop=F], names=methods)
   title(paste("Query=", querytype, "Var= ", variable, "Row= ", row))
}
barchart.method <- 
   function(querytype="all",
            variable="goodness",
            database= clustsys.data,
            method="e"
           ) {
   barplot(t(database[[querytype]][[variable]][,method,drop=F]), 
                names=exp.param$system.abbr)
   title(paste("Query=", querytype, "Var= ", variable, "Method= ", method))
}


piechart <- 
    function(querytype= "all",
             variable="goodness",
            database= clustsys.data
         )
{
#   methods <- dimnames(database[[querytype]][[variable]])[[2]]
   pie(database[[querytype]][[variable]])
   title(paste("Query=", querytype, "Var= ", variable))
}
dot.plot.method <-
   function(querytype="all",
            variable="goodness",
            database= clustsys.data,
            method= "e",
            ...
           ) {
   plot(database[[querytype]][[variable]][,method],
        xlab = "System id",
        ylab = paste(variable, "value"),
        ...)
        title(paste("Query=", querytype, "Var= ", variable, "Method= ", method))

}


dot.plot.row <-
   function(querytype="all",
            variable="goodness",
            database= clustsys.data,
            row= "cs453110",
            ...
           ) {
   plot(database[[querytype]][[variable]][row,], 
        xlab = "System id",
        ylab = paste(variable, "value"),
        ...)
        title(paste("Query=", querytype, "Var= ", variable, "Row= ", row))

}


plot.variable <- function(
       database=clustsys.data,
       methods=exp.param[["clustering.methods"]],
       variable = "system.goodness",
       query="all",
       type="b",
       xtck=T,
       cex=1.5,
       x.legend=4.6,
       y.legend=0.63,
       mar=c(5,5.5,0,1)+0.6,
       select.programs = database$select.programs,
       ...
      )
{

   par.save <- par()

  # The actual matrix to be plotted
     v <- extract.across(variable=variable, database=database)[[query]]

  # number of programs for which information is plotted
    num.programs <- nrow(v)

  # d.p is programs to be displayed. Only programs mentioned in
  # exp.param$select.programs are used in the analysis. All else are dropped.

    d.p <- select.programs

  # get x and y label messages
     x.label <- "Program number"
     y.label <- database[["variables.print.name"]][variable]

  # Change the margins to accomodate the labels

   if (cex > 1) {
      par(mar=mar)
   }

  # get the minimum and maximum of this plot .. to draw the vertical tick line
    y.min <- min(v[d.p,methods])
    y.max <- max(v[d.p,methods])

   # Define the plot region and axes
   tsplot(c(y.min, y.max, rep(y.min, length(d.p)-2)), type="n", lty=1, pch=1,
           ylab= "", xlab="",  ...)
   title(main="", ylab= y.label, xlab=x.label, cex=cex)


   # draw vertical tick line if requested
   if (xtck) {
       x1 <- 1:length(d.p)
       y1 <- rep(y.min+0.01, length(d.p))
       y2 <- rep(y.max, length(d.p))
       segments(x1, y1, x1, y2, lty=2, xaxt="d", yaxt="d", lwd=0.1)
   }

  # number of methods for which the variable is plotted
    methods.num <- length(methods)

   # draw the legend box. The placement was found by trial and error
   # legend should be drawn after the vertical ticks
   # So that it overwrites the overlapping region...

  # names of methods to be printed in legend
     legend.text <- database[["methods.print.name"]][methods]

     legend(x.legend, y.legend, legend.text, lty=lty.set[1:methods.num], cex=cex)

  # method number in methods list
    i <- 1

  # now plot the variables for the other methods

    par(yaxs="d", xaxs="d")
     for (method in methods) {
        par(new=T)
        plot(v[d.p,method], type=type, pch=i, lty=lty.set[i], xlab="", ylab="", axes=F, ...)
        i <- i+1
    }

  # restore status of all the par variables
   par(par.save)
   par(new=F)

  NULL
}


dump.screen <- function(file,
                        width=6,
                        height=6*0.75,
                        ...) {
        dev.print(postscript, file=file, width=width, height=height, ...)
}

dump.hist  <- function(width=6, height=6*0.75, ...) {
     file<- hist.filename(...)
     print(paste("Dumping the histogram into ", file, sep=""))
     dump.screen(file, width=width, height=height)
}


hist.filename <- function(database=clustsys.data,
                               querytype="all",
                               method="e",
                               variable="system.goodness",
                               dir=output.dir,
                               ...) {

  f<- paste("hist", database$year, "-", querytype, "-", method, "-", variable,
              sep="")
    
  paste(dir, "/", f, ".eps", sep="")
}
