!! in-package("CREW")
!! in-grammar('user)

#||
------------ RCS comments ------------
/home/bayou1/lakhotia/arun/CVSROOT/Clustsys/src/refine/clustsys.re,v 1.1.1.1 1995/07/28 21:22:22 arun Exp
arun
1995/07/28 21:22:22
1.1.1.1

clustsys.re,v
# Revision 1.1.1.1  1995/07/28  21:22:22  arun
# Subsystem Classification
#
# Revision 1.7  1994/07/09  07:56:05  jmg
# Changed call to parse-program to trap-errors.
# Added code to print parse results.
# These changes extend the buggy C code we can handle.
#
# Revision 1.6  1994/07/05  22:45:16  jmg
# Finished breaking up into general use and specific use functions.
# Finished cleaning up code.
#
# Revision 1.5  1994/07/02  02:47:15  jmg
# Added numerous small functions to break up existing functions.
# Added PRINT-ALL-FUNCS to print all function names, file, and static?.
# Changed all computed-using constructs (mostly to set formers).
# Added/changed Refine type docs to each function.
# Changed program logic so that call to parse-program is independent
# 	of a query or a print or whatever.
#
# Revision 1.4  1994/07/01  00:21:38  jmg
# Removed extraneous code
#
# Revision 1.3  1994/06/29  03:20:36  jmg
# Added new functions
# 	CHECK-SCOPE-SET
# 	CHECK-MEMBER-CLASS
# Fixed to properly exit when bad queery value passed in.
#
# Revision 1.2  1994/06/02  03:06:01  jmg
# Changed to accept string for query type instead of character.
#
# Revision 1.1  1994/06/02  02:58:18  jmg
# Initial revision
#
------------ end of RCS comments ------------
||#
#||
-------------------------------------------------------------------------------------
			Cluster Analysis System

This group of functions performs the front-end of cluster analysis for C programs.

The relationship used is def-use.
-------------------------------------------------------------------------------------
||#

%----- TYPES -----
type myset = set(tuple(func:      identifier, 
		       field-var: identifier
		       ))
type def-use-info-set = set(tuple(def-func: identifier, 
				  use-func: identifier,
				  var-name: identifier
				  ))

%----- CONSTANTS -----
constant *LEGAL-QUERY-SET* : set(string) = { "all", "field", "globals" }

%----- VARIABLES -----
var *PARSED-PROGRAM* : cls::program = NIL
var *PARSED-PROGRAM-FILES* : cls::program = NIL

#||
%THIS COULD BE DONE TO REPORT ON A PER FILE BASIS INSTEAD OF WHEN DONE IF I COULD
%FIGURE OUT HOW TO KNOW THE FILE THAT WAS JUST PARSED -- JMG, 7/8/94
%----------- set the function(s) to be called after parsing each file --------------
%var cls::*AFTER-PARSING-FILE-FUNCTIONS* : seq(symbol) = [ 'report-parse-results ]
||#

%====================================================================================
%			GENERAL USE FUNCTIONS
%====================================================================================

"This function is used to check a passed-in scope (anint) to see if it is in the set of 
passed-in scopes (scope-set).  Scope-set is  meant to be a set of Refine/C 
\"identifier-scope-depths\" .  The possible depths are (from p. 3-27, Refine/C Programmer's 
Guide):
	1) Global 		-- value is 0 (zero)
	2) Formal parameters 	-- value is 1
	3) Local variables	-- values start at 2"
function CHECK-SCOPE-SET(anint: integer,
			 scope-set: set(integer)) : boolean =
           empty(scope-set)
   or-else anint in scope-set

%====================================================================================

"This function is used to check a symbol (sym) and see if it is in the set of symbols
(filter-class).  Filter-class is meant to be a set of Refine/C \"identifier-classes\".
The possible classes are (from p. 3-26, Refine/C Programmer's Guide):
	1) Function
	2) Variable
	3) Typedef
	4) Struct-tag
	5) Enum-tag
	6) Union-tag
	7) Element
	8) Label
	9) Enumeral"
function CHECK-MEMBER-CLASS(sym: symbol,
			    filter-class: set(symbol)) : boolean =
           empty(filter-class)
   or-else sym in filter-class

%====================================================================================

"This function is used to determine if an object is set (i.e., defined)."
function IS-REF-SET?(a-ref: object) : boolean =
   identifier-ref-usage(a-ref) = 'set

%====================================================================================

"This function is used to determine if an object is  updated (i.e., assigned)."
function IS-REF-UPDATED?(a-ref: object) : boolean =
   identifier-ref-usage(a-ref) = 'update

%====================================================================================

"This function is used to determine if an object is used."
function IS-REF-USED?(a-ref: object) : boolean =
   identifier-ref-usage(a-ref) = 'use

%====================================================================================

"This fu2nction is used to determine if an object is set or updated (i.e., defined)."
function IS-REF-SET-OR-UPDATED?(a-ref: object) : boolean =
           is-ref-set?(a-ref)
   or-else is-ref-updated?(a-ref)

%====================================================================================

"This function returns whether an identifier's storage class is static."
function IS-ID-STATIC?(id: identifier) : boolean =
   identifier-storage-class(id) = 'static

%====================================================================================

"This function opens a file on disk for text output."
function OPEN-DISK-FILE-FOR-TEXT-OUTPUT(fname: string) : any-type =
   open(fname, 
	'::direction, '::output, 
	'::element-type, '::default,	
	'::if-exists, '::overwrite, 
	'::if-does-not-exist, '::create
        )

%====================================================================================

"This function converts an identifier object to the string representation 
of its name."
function ID-TO-STRING(id: identifier) : string =
   symbol-to-string(id-name(id))

%====================================================================================

"This function converts a file to the string representation of its name."
function FILE-TO-FILE-NAME(afile: file) : string =
   file-spec-name(file-to-file-spec(afile))

%====================================================================================

"This function converts an identifier to the string representation of its file name."
function ID-TO-FILE-NAME(id: identifier) : string =
   file-to-file-name(identifier-file(id))

%====================================================================================

"This function determines if a file parsed correctly."
function FILE-PARSED-OK?(f: file-spec) : boolean =
   undefined?(file-spec-parse-error(f))

%====================================================================================

"This function is called after the program is parsed.  It prints whether each file
properly parsed."
function REPORT-PARSE-RESULTS(prg: program) =
   let(fname   :string  = "",
       ferror  :string  = "",
       ferrline:integer = 0
       )
   (enumerate f over program-file-specs(prg) do
       fname <- file-spec-name(f);
       format(true, "Parse of ~A ", fname);
       (if file-parsed-ok?(f) then
           format(true, "ok.~%")
        else
           ferror   <- file-spec-parse-error(f);
           ferrline <- file-spec-line-number(f);
           format(true, "failed on line ~A.~%", ferrline);
           format(true, "Error was: ~A~%", ferror)
        )
    )

%====================================================================================

"This function parses the program passed in to it as the name of a \".prog\" file and
store the results in the global *PARSED-PROGRAM*."
function PARSE-THE-PROGRAM(program-file: string) =
   *PARSED-PROGRAM-FILES* <- cls::parse-program-file(program-file);
   *PARSED-PROGRAM* <- cls::parse-program(*PARSED-PROGRAM-FILES*, 
                                          true,			% trap errors
                                          '::detach-asts?, NIL);
   report-parse-results(*PARSED-PROGRAM*)

%====================================================================================

"This function is used to get the set of function identifiers contained in a program
and return it as a sequence."
function GET-FUNCTION-IDS(prg: cls::program) : seq(cls::identifier) =
   [ fcn-ID | (fcn-ID: identifier, f: object) 
          program(prg)
     	& f IN program-files(prg)
     	& fcn-ID IN file-globals(f)
     	& Identifier-Class(fcn-ID) = 'function
    ]

%====================================================================================

"This function is used to get the set of function identifier references in a program
and return it."
function SET-OF-FUNCTION-REFS(fcn-ID: cls::identifier) : set(object) =
   if identifier-class(fcn-ID) = 'function then
      identifier-contained-refs(fcn-ID)
   else
      {}

%====================================================================================

"This function returns the set of function identifiers setting variable identifiers."
function IDENTIFIERS-SET(prg: cls::Program,
			 filter-class: set(symbol),
			 scope-set: set(integer)) : myset =
   { < fcn-ID, var-ID > | (fcn-ID: identifier, var-ID: identifier)
     	  program(prg)
        & fcn-ID in get-function-ids(prg)
   	& var-ID in identifiers-set-by-a-function(fcn-ID, filter-class, scope-set)
   	& defined?(var-ID)
    }

%====================================================================================

"This function returns the set of variable identifiers set by the passed in function
identifier."
function IDENTIFIERS-SET-BY-A-FUNCTION(fcn-ID: cls::identifier,
				       filter-class: set(symbol),
				       scope-set: set(integer)) : set(cls::identifier) =
   { var-ID | (var-ID: identifier, a-ref: object)
     	  a-ref in set-of-function-refs(fcn-ID)
   	& is-ref-set-or-updated?(a-ref)
   	& Ref-To(a-ref) = var-ID
   	& defined?(var-ID)
   	& check-member-class(Identifier-Class(var-ID), filter-class) 
   	& check-scope-set(identifier-scope-depth(var-ID), scope-set)
    }

%====================================================================================

"This function returns the set of function identifiers setting using variable
identifiers."
function IDENTIFIERS-USED(prg: cls::Program,
			  filter-class: set(symbol),
			  scope-set: set(integer)) : myset =
   { < fcn-ID, var-ID > | (fcn-Id: identifier, var-ID: identifier)
     	  program(prg)
   	& fcn-ID in get-function-ids(prg)
   	& var-ID in identifiers-used-by-a-function(fcn-ID, filter-class, scope-set)
   	& defined?(var-ID)
    }

%====================================================================================

"This function returns the set of variable identifiers used by the passed in function
identifier."
function IDENTIFIERS-USED-BY-A-FUNCTION(fcn-ID: cls::identifier,
					filter-class: set(symbol),
					scope-set: set(integer)) : set(cls::identifier) =
   { var-ID | (var-ID: identifier, a-ref: identifier)
     	  a-ref in set-of-function-refs(fcn-ID)
   	& is-ref-used?(a-ref)
   	& Ref-To(a-ref) = var-ID
   	& defined?(var-ID)
   	& check-member-class(Identifier-Class(var-ID), filter-class)
   	& check-scope-set(identifier-scope-depth(var-ID), scope-set)
    }

%====================================================================================
%			SPECIFIC USE FUNCTIONS
%====================================================================================

"This function prints the results of the passed in clustsys query into a file
whose name is passed in."
function PRINT-DEF-USE-INFO-SET(fname: string,
				S3: def-use-info-set) =
   let(def-func: string = "",
       use-func: string = "", 
       var-name : string = "",
       ofile = open-disk-file-for-text-output(fname)
       )
     (
      (enumerate f over S3 do
         def-func  <- id-to-string(f.def-func);
         use-func  <- id-to-string(f.use-func);
         var-name  <- id-to-string(f.var-name);
         format(ofile, "~25A  ~25A  ~25A~%", def-func, use-func, var-name)
       );
      close(ofile)
      )

%====================================================================================

"This function performs a clustsys query on an already parsed program."
function CLUSTSYS-QUERY(fname: string,
                        query-type: string) =
   if query-type in *LEGAL-QUERY-SET* then
     (let (S1: myset = {},
           S2: myset = {},
           filter-class: set(symbol) = {},
	   scope-set: set(integer) = {}
           )
     (if query-type = "field" then
        filter-class <- {'element}
      elseif query-type = "globals" then
        filter-class <- {'variable};
        scope-set    <- {0}
      %elseif query-type = "all" then 		% NO WORK TO DO
      );
      format(true, "Done running cluster-~A-query.~%", query-type);
      S1 <- identifiers-set(*PARSED-PROGRAM*, filter-class, scope-set);
      S2 <- identifiers-used(*PARSED-PROGRAM*, filter-class, scope-set);
      (let (S3: def-use-info-set = cluster-join(S1, S2))
       print-def-use-info-set(fname, S3)
       )
      )
    else
      format(true, "Bad value for query selector.~%")

%====================================================================================

"This function performs a join into a three tuple from two sets of two tuples such
that the second \"field\" of the two tuples is the same and the first \"field\" of
the two tuples is different."
function CLUSTER-JOIN(S1: myset,
		      S2: myset) : def-use-info-set =
   { < x, y, z > | (x: identifier, y: identifier, z: identifier) 
          < x, z > in S1 
    	& < y, z > in S2 
    	& id-name(x) ~= id-name(y) 
    }

%====================================================================================

"This function prints the name of all the functions in an already parsed program,
the file they are in, and whether the function is static or not into a file whose
name is passed in."
function PRINT-ALL-FUNCS(fname: string) =
   let(the-func:   string = "",
       the-file:   string = "", 
       the-static: string = "",
       all-funcs:  seq(cls::identifier) = get-function-ids(*PARSED-PROGRAM*),
       ofile = open-disk-file-for-text-output(fname)
       )
     (
       (enumerate f over all-funcs do
          the-func    <- id-to-string(f);
          the-file    <- id-to-file-name(f);
          (if is-id-static?(f) then
             the-static  <- "static"
           else
             the-static  <- ""
          );
          format(ofile, "~25A  ~25A  ~25A~%", the-func, the-file, the-static)
        );
       close(ofile)
      )

%====================================================================================
% end of clustsys.re
