!! in-package("CREW")
!! in-grammar('user)

%----- VARIABLES -----
var *PARSED-PROGRAM* : rcu::program = NIL
var *PARSED-PROGRAM-FILES* : rcu::program = NIL

%====================================================================================


function CREATE-PROG-FILE (primary: string, directory: string) :string =

% A complete filename is considered as "dir/primary.extension"

   let (fileprefix   : string = undefined,
	c-source-file: string = undefined,
	prog-file    : string = undefined,
	command      : string = undefined)

     fileprefix  <- concat(directory,"/", primary);
     prog-file   <- concat(fileprefix, ".prog");
     c-source-file <- concat(fileprefix, ".c");

     command     <- concat("make-C-program ", primary,
                           " -F ", c-source-file,
		           " -dir ", directory, " > ", prog-file);

     excl::run-shell-command(command);
     prog-file





%====================================================================================

"This function determines if a file parsed correctly."
function FILE-PARSED-OK?(f: rcu::file-spec) : boolean =
   undefined?(rcu::file-spec-parse-error(f))

%====================================================================================

"This function is called after the program is parsed.  It prints whether each file
properly parsed."
function REPORT-PARSE-RESULTS(prg: rcu::program) =
   let(fname   :string  = "",
       ferror  :string  = "",
       ferrline:integer = 0
       )
   (if defined?(rcu::program-file-specs(prg)) then
    enumerate f over rcu::program-file-specs(prg) do
       fname <- rcu::file-spec-name(f);
       format(true, "Parse of ~A ", fname);
       (if file-parsed-ok?(f) then
           format(true, "ok.~%")
        else
           ferror   <- rcu::file-spec-parse-error(f);
           ferrline <- rcu::file-spec-line-number(f);
           format(true, "failed on line ~A.~%", ferrline);
           format(true, "Error was: ~A~%", ferror)
        )
    )

%====================================================================================

"Parse a single C program. Should be called with only the primary name
 of the file, i.e. the .c extension should be omitted. Further, the C
 file is expected to be in the current directory."

function PARSE-C-FILE(primary-name: string) =
   parse-the-program(create-prog-file(primary-name, "."))

%====================================================================================

"This function parses the program passed in to it as the name of a
 \".prog\" file and store the results in the global *PARSED-PROGRAM*."

function PARSE-THE-PROGRAM(program-file: string) : rcu::program =
   *PARSED-PROGRAM-FILES* <- rcu::parse-program-file(program-file);
   *PARSED-PROGRAM* <- rcu::parse-program(*PARSED-PROGRAM-FILES*, 
                                          true,			% trap errors
                                          '::detach-asts?, NIL);
%   report-parse-results(*PARSED-PROGRAM*);
   *PARSED-PROGRAM*

%====================================================================================

function LIST-PROGRAM(pgm: rcu::program) =
   enumerate a-file over rcu::program-files(pgm) do
         list-file-contents(a-file)

function LIST-FILE-CONTENTS(a-file: rcu::file) =
     format(true, "File: ~A ~&", rcu::file-pathname(a-file));
     enumerate def-in-file over rcu::file-definitions(a-file) do
       if rcu::include-statement(def-in-file) then
         format(true, "Include: ~A ~&", rcu::included-file-name(def-in-file))
       elseif rcu::declaration(def-in-file) then
         enumerate decl over rcu::declaration-declarators(def-in-file) do
                  format(true, "~\\pp\\~%", decl)
       else
         format(true, "Function def: ~A~&",
		rcu::id-name(rcu::function-identifier(def-in-file)))

