!! in-package("CREW")
!! in-grammar('user)

#||
------------ RCS comments ------------
/home/bayou1/lakhotia/arun/CVSROOT/Clustsys/src/refine/refine-queries.re,v 1.2 1995/11/06 19:24:12 pjm9806 Exp
pjm9806
1995/11/06 19:24:12
1.2

refine-queries.re,v
# Revision 1.2  1995/11/06  19:24:12  pjm9806
# Added package prefix to refine code.
#
# Changed cls prefix to new rcu prefix.
#
# Revision 1.1.1.1  1995/07/28  21:22:23  arun
# Subsystem Classification
#
# Revision 1.7  1994/07/09  07:56:05  jmg
# Changed call to parse-program to trap-errors.
# Added code to print parse results.
# These changes extend the buggy C code we can handle.
#
# Revision 1.6  1994/07/05  22:45:16  jmg
# Finished breaking up into general use and specific use functions.
# Finished cleaning up code.
#
# Revision 1.5  1994/07/02  02:47:15  jmg
# Added numerous small functions to break up existing functions.
# Added PRINT-ALL-FUNCS to print all function names, file, and static?.
# Changed all computed-using constructs (mostly to set formers).
# Added/changed Refine type docs to each function.
# Changed program logic so that call to parse-program is independent
# 	of a query or a print or whatever.
#
# Revision 1.4  1994/07/01  00:21:38  jmg
# Removed extraneous code
#
# Revision 1.3  1994/06/29  03:20:36  jmg
# Added new functions
# 	CHECK-SCOPE-SET
# 	CHECK-MEMBER-CLASS
# Fixed to properly exit when bad queery value passed in.
#
# Revision 1.2  1994/06/02  03:06:01  jmg
# Changed to accept string for query type instead of character.
#
# Revision 1.1  1994/06/02  02:58:18  jmg
# Initial revision
#
------------ end of RCS comments ------------
||#
#||
-------------------------------------------------------------------------------------
			Cluster Analysis System

This group of functions performs the front-end of cluster analysis for C programs.

The relationship used is def-use.
-------------------------------------------------------------------------------------
||#

%----- TYPES -----
type myset = set(tuple(func:      identifier, 
		       field-var: identifier
		       ))
type def-use-info-set = set(tuple(def-func: identifier, 
				  use-func: identifier,
				  var-name: identifier
				  ))

%----- CONSTANTS -----
constant *LEGAL-QUERY-SET* : set(string) = { "all", "field", "globals" }


%====================================================================================
%			GENERAL USE FUNCTIONS
%====================================================================================

"This function is used to check a passed-in scope (anint) to see if it is in the set of 
passed-in scopes (scope-set).  Scope-set is  meant to be a set of Refine/C 
\"identifier-scope-depths\" .  The possible depths are (from p. 3-27, Refine/C Programmer's 
Guide):
	1) Global 		-- value is 0 (zero)
	2) Formal parameters 	-- value is 1
	3) Local variables	-- values start at 2"
function CHECK-SCOPE-SET(anint: integer,
			 scope-set: set(integer)) : boolean =
           empty(scope-set)
   or-else anint in scope-set

%====================================================================================

"This function is used to check a symbol (sym) and see if it is in the set of symbols
(filter-class).  Filter-class is meant to be a set of Refine/C \"identifier-classes\".
The possible classes are (from p. 3-26, Refine/C Programmer's Guide):
	1) Function
	2) Variable
	3) Typedef
	4) Struct-tag
	5) Enum-tag
	6) Union-tag
	7) Element
	8) Label
	9) Enumeral"
function CHECK-MEMBER-CLASS(sym: symbol,
			    filter-class: set(symbol)) : boolean =
           empty(filter-class)
   or-else sym in filter-class

%====================================================================================

"This function is used to determine if an object is set (i.e., defined)."
function IS-REF-SET?(a-ref: object) : boolean =
   identifier-ref-usage(a-ref) = 'rcu::set

%====================================================================================

"This function is used to determine if an object is  updated (i.e., assigned)."
function IS-REF-UPDATED?(a-ref: object) : boolean =
   identifier-ref-usage(a-ref) = 'rcu::update

%====================================================================================

"This function is used to determine if an object is used."
function IS-REF-USED?(a-ref: object) : boolean =
   identifier-ref-usage(a-ref) = 'rcu::use

%====================================================================================

"This fu2nction is used to determine if an object is set or updated (i.e., defined)."
function IS-REF-SET-OR-UPDATED?(a-ref: object) : boolean =
           is-ref-set?(a-ref)
   or-else is-ref-updated?(a-ref)

%====================================================================================

"Test if an identifier's storage class is static."
function IS-ID-STATIC?(id: identifier) : boolean =
 if defined?(identifier-storage-class(id)) then
   identifier-storage-class(id) = 'rcu::static
 elseif defined?(identifier-links(id)) then
   ex(x) (x in identifier-links(id) & identifier-storage-class(x) = 'rcu::static)
 else false

%====================================================================================

"This function opens a file on disk for text output."
function OPEN-DISK-FILE-FOR-TEXT-OUTPUT(fname: string) : any-type =
  if fname = "" then
    *standard-output*
  else
   open(fname, 
	'::direction, '::output, 
	'::element-type, '::default,	
	'::if-exists, '::overwrite, 
	'::if-does-not-exist, '::create
        )

function CLOSE-DISK-OUTPUT-FILE (ofile: any-type) =
    if ~ re::default-standard-output-stream?(ofile) then
       close(ofile)


%====================================================================================

"This function converts an identifier object to the string representation 
of its name."
function ID-TO-STRING(id: identifier) : string =
  if is-id-static?(id) 
  then
       % create a unique name when an ID is static
   concat(symbol-to-string(id-name(id)),
          "-",
          symbol-to-string(file-name(identifier-file(id))))

  else
   symbol-to-string(id-name(id))

%====================================================================================

"This function converts a file to the string representation of its name."
function FILE-TO-FILE-NAME(afile: file) : string =
   file-spec-name(file-to-file-spec(afile))

%====================================================================================

"This function converts an identifier to the string representation of its file name."
function ID-TO-FILE-NAME(id: identifier) : string =
   file-to-file-name(identifier-file(id))

%====================================================================================
"This function is used to get the set of function identifiers contained in a program
and return it as a sequence."
function GET-FUNCTION-IDS(prg: rcu::program) : seq(rcu::identifier) =
   [ fcn-ID | (fcn-ID: identifier, f: object) 
          program(prg)
     	& f IN program-files(prg)
     	& fcn-ID IN file-globals(f)
     	& Identifier-Class(fcn-ID) = 'rcu::function
    ]

"Finds the IDENTIFIER for the function FUNCTION-NAME in program PRG."
function FIND-FUNCTION-ID(prg: program, function-name : symbol) : identifier 
    computed-using
          program(prg)
     	& f in program-files(prg)
     	& fcn-ID in file-globals(f)
     	& Identifier-Class(fcn-ID) = 'rcu::function
        & Id-name(fcn-ID) = function-name
        => find-function-id(prg, function-name) = fcn-ID

"Prints a table of IDENTIFIER referred in FUNCTION-NAME. Also prints
 their usage information (i.e., SET, USE, or UPDATE)."
function SHOW-CONTAINED-REFS(function-name : symbol) =
   let (fcn-ID = find-function-id(*parsed-program*, function-name))
   if undefined?(fcn-ID) then
       format(true, "Function ~A not present in current program~%", function-name)
   else
      enumerate ref over identifier-contained-refs(fcn-ID) do
        format(true, "~25A ~A~%", id-name(ref), identifier-ref-usage(ref))

%====================================================================================

"This function is used to get the set of function identifier references in a program
and return it."
function SET-OF-FUNCTION-REFS(fcn-ID: rcu::identifier) : set(object) =
   if identifier-class(fcn-ID) = 'rcu::function then
      identifier-contained-refs(fcn-ID)
   else
      {}

%====================================================================================

"This function returns the set of function identifiers setting variable identifiers."
function IDENTIFIERS-SET(prg: rcu::Program,
			 filter-class: set(symbol),
			 scope-set: set(integer)) : myset =
   { < fcn-ID, var-ID > | (fcn-ID: identifier, var-ID: identifier)
     	  program(prg)
        & fcn-ID in get-function-ids(prg)
   	& var-ID in identifiers-set-by-a-function(fcn-ID, filter-class, scope-set)
   	& defined?(var-ID)
    }

%====================================================================================

"This function returns the set of variable identifiers set by the passed in function
identifier."
function IDENTIFIERS-SET-BY-A-FUNCTION(fcn-ID: rcu::identifier,
				       filter-class: set(symbol),
				       scope-set: set(integer)) : set(rcu::identifier) =
   { var-ID | (var-ID: identifier, a-ref: object)
     	  a-ref in set-of-function-refs(fcn-ID)
   	& is-ref-set-or-updated?(a-ref)
   	& Ref-To(a-ref) = var-ID
   	& defined?(var-ID)
   	& check-member-class(Identifier-Class(var-ID), filter-class) 
   	& check-scope-set(identifier-scope-depth(var-ID), scope-set)
    }

%====================================================================================

"This function returns the set of function identifiers setting using variable
identifiers."
function IDENTIFIERS-USED(prg: rcu::Program,
			  filter-class: set(symbol),
			  scope-set: set(integer)) : myset =
   { < fcn-ID, var-ID > | (fcn-Id: identifier, var-ID: identifier)
     	  program(prg)
   	& fcn-ID in get-function-ids(prg)
   	& var-ID in identifiers-used-by-a-function(fcn-ID, filter-class, scope-set)
   	& defined?(var-ID)
    }

%====================================================================================

"This function returns the set of variable identifiers used by the passed in function
identifier."
function IDENTIFIERS-USED-BY-A-FUNCTION(fcn-ID: rcu::identifier,
					filter-class: set(symbol),
					scope-set: set(integer)) : set(rcu::identifier) =
   { var-ID | (var-ID: identifier, a-ref: identifier)
     	  a-ref in set-of-function-refs(fcn-ID)
   	& is-ref-used?(a-ref)
   	& Ref-To(a-ref) = var-ID
   	& defined?(var-ID)
   	& check-member-class(Identifier-Class(var-ID), filter-class)
   	& check-scope-set(identifier-scope-depth(var-ID), scope-set)
    }

%====================================================================================

"Returns set of tuples of identifiers referred by functions in the
 program. The set is filtered based on contraints stated by the other
 parameters."

function IDS-REFERRED(prg: rcu::Program,
                      usage-class: set(symbol),
                      filter-class: set(symbol),
		      scope-set: set(integer)) : myset =
   { < fcn-ID, var-ID > | (fcn-ID: identifier, var-ID: identifier)
     	  program(prg)
        & fcn-ID in get-function-ids(prg)
   	& var-ID in ids-referred-by-a-function(fcn-ID, usage-class, filter-class, scope-set)
   	& defined?(var-ID)
    }

%====================================================================================

"Returns the set of identifiers referred by a function. The set is 
 filtered based on the usage-class, filter-class, and scope-set contraints."
function IDS-REFERRED-BY-A-FUNCTION(fcn-ID: rcu::identifier,
                             usage-class : set(symbol),
			     filter-class: set(symbol),
			     scope-set: set(integer)) : set(rcu::identifier) =
   { var-ID | (var-ID: identifier, a-ref: object)
     	  a-ref in set-of-function-refs(fcn-ID)
   	& check-member-class(identifier-ref-usage(a-ref), usage-class)
   	& Ref-To(a-ref) = var-ID
   	& defined?(var-ID)
   	& check-member-class(Identifier-Class(var-ID), filter-class) 
   	& check-scope-set(identifier-scope-depth(var-ID), scope-set)
    }
%====================================================================================
%			SPECIFIC USE FUNCTIONS
%====================================================================================

"This function prints the results of the passed in clustsys query into a file
whose name is passed in."
function PRINT-DEF-USE-INFO-SET(fname: string,
				S3: def-use-info-set) =
   let(def-func: string = "",
       use-func: string = "", 
       var-name : string = "",
       ofile = open-disk-file-for-text-output(fname)
       )
     (
      (enumerate f over S3 do
         def-func  <- id-to-string(f.def-func);
         use-func  <- id-to-string(f.use-func);
         var-name  <- id-to-string(f.var-name);
         format(ofile, "~25A  ~25A  ~A~%", def-func, use-func, var-name)
       );
      close-disk-output-file(ofile)
      )

%====================================================================================

"This function performs a clustsys query on an already parsed program."
function CLUSTSYS-QUERY(fname: string,
                        query-type: string) =
   if query-type in *LEGAL-QUERY-SET* then
     (let (S1: myset = {},
           S2: myset = {},
           S1-usage-class: set(symbol) = {'rcu::set},
           S2-usage-class: set(symbol) = {'rcu::use},
           filter-class:   set(symbol) = {},
	   scope-set:      set(integer) = {}
           )
     (if query-type = "field" then
        filter-class   <- {'rcu::element}
      elseif query-type = "globals" then
        filter-class   <- {'rcu::variable};
        scope-set      <- {0}
      else   % query-type = "all" then 		% NO WORK TO DO
        S1-usage-class <- {};
        S2-usage-class <- {} 
      );
      format(true, "Done running cluster-~A-query.~%", query-type);
      S1 <- ids-referred(*PARSED-PROGRAM*, S1-usage-class, filter-class, scope-set);
      (if (S1-usage-class ~= S2-usage-class) then
        S2 <- ids-referred(*PARSED-PROGRAM*, S2-usage-class, filter-class, scope-set)
       else S2 <- S1);
      (let (S3: def-use-info-set = cluster-join(S1, S2))
       print-def-use-info-set(fname, S3)
       )
      )
    else
      format(true, "Bad value for query selector.~%")

%====================================================================================

"This function performs a join into a three tuple from two sets of two tuples such
that the second \"field\" of the two tuples is the same and the first \"field\" of
the two tuples is different."
function CLUSTER-JOIN(S1: myset,
		      S2: myset) : def-use-info-set =
   { < x, y, z > | (x: identifier, y: identifier, z: identifier) 
          < x, z > in S1 
    	& < y, z > in S2 
    	& id-name(x) ~= id-name(y) 
    }

%====================================================================================

"This function prints the name of all the functions in an already parsed program,
the file they are in, and whether the function is static or not into a file whose
name is passed in."
function PRINT-ALL-FUNCS(fname: string) =
   let(the-func:   string = "",
       the-file:   string = "", 
       the-static: string = "",
       all-funcs:  seq(rcu::identifier) = get-function-ids(*PARSED-PROGRAM*),
       ofile = open-disk-file-for-text-output(fname)
       )
    let (lisp::*standard-output* = ofile)
       (
       (enumerate f over all-funcs do
          the-func    <- id-to-string(f);
          the-file    <- id-to-file-name(f);
          (if is-id-static?(f) then
             the-static  <- "static"
           else
             the-static  <- ""
          );
%          format(ofile, "~25A  ~A  ~A~%", the-func, the-file, the-static)
          format(true, "~25A  ~25A  ~25A~%", the-func, the-file, the-static)
        );
       close-disk-output-file(ofile)
      )

function PRINT-TEXT(fname: string, message: string) =
   let(ofile = open-disk-file-for-text-output(fname))
   let (*standard-output* = ofile)
%          format(ofile, "~25A %", message)
      format(true, "~25A ~%", message);
      close-disk-output-file(ofile)


%====================================================================================
% end of clustsys.re
